searchNodes=[{"doc":"Timex Timex is a rich, comprehensive Date/Time library for Elixir projects, with full timezone support via the :tzdata package. If you need to manipulate dates, times, datetimes, timestamps, etc., then Timex is for you! It is very easy to use Timex types in place of default Erlang types, as well as Ecto types via the timex_ecto package.The complete documentation for Timex is located here.Migrating to Timex 3.xSee the Migrating section further down for details. If you are migrating from earlier than 2.x, please review this migration doc for 1.x to 2.x.Timex 3.0 is a significant rewrite, in order to accommodate Elixir 1.3's new Calendar types in a semantically correct way. The external API is mostly the same, but there are changes, many without deprecations, so please read the changelog carefully.Getting StartedThere are some brief examples on usage below, but I highly recommend you review the API docs here, there are many examples, and some extra pages with richer documentation on specific subjects such as custom formatters/parsers, etc.Adding Timex To Your ProjectTo use Timex with your projects, edit your mix.exs file and add it as a dependency:defp deps do [{:timex, &quot;~&gt; 3.6&quot;}] endQuickfast introductionTo use Timex, I recommend you add use Timex to the top of the module where you will be working with Timex modules, all it does is alias common types so you can work with them more comfortably. If you want to see the specific aliases added, check the top of the Timex module, in the __using__/1 macro definition.Here's a few simple examples:&gt; use Timex &gt; Timex.today ~D[2016-02-29] &gt; datetime = Timex.now #&lt;DateTime(2016-02-29T12:30:30.120+00:00Z Etc/UTC) &gt; Timex.now(&quot;America/Chicago&quot;) #&lt;DateTime(2016-02-29T06:30:30.120-06:00 America/Chicago) &gt; Duration.now #&lt;Duration(P46Y6M24DT21H57M33.977711S)&gt; &gt; {:ok, default_str} = Timex.format(datetime, &quot;{ISO:Extended}&quot;) {:ok, &quot;2016-02-29T12:30:30.120+00:00&quot;} &gt; {:ok, relative_str} = Timex.shift(datetime, minutes: -3) |&gt; Timex.format(&quot;{relative}&quot;, :relative) {:ok, &quot;3 minutes ago&quot;} &gt; strftime_str = Timex.format!(datetime, &quot;%FT%T%:z&quot;, :strftime) &quot;2016-02-29T12:30:30+00:00&quot; &gt; Timex.parse(strftime_str, &quot;{ISO:Extended}&quot;) {:ok, #&lt;DateTime(2016-02-29T12:30:30.120+00:00 Etc/Utc)} &gt; Timex.parse!(strftime_str, &quot;%FT%T%:z&quot;, :strftime) #&lt;DateTime(2016-02-29T12:30:30.120+00:00 Etc/Utc) &gt; Duration.diff(Duration.now, Duration.zero, :days) 16850 &gt; Timex.shift(date, days: 3) ~D[2016-03-03] &gt; Timex.shift(datetime, hours: 2, minutes: 13) #&lt;DateTime(2016-02-29T14:43:30.120Z Etc/UTC)&gt; &gt; timezone = Timezone.get(&quot;America/Chicago&quot;, Timex.now) #&lt;TimezoneInfo(America/Chicago - CDT (-06:00:00))&gt; &gt; Timezone.convert(datetime, timezone) #&lt;DateTime(2016-02-29T06:30:30.120-06:00 America/Chicago)&gt; &gt; Timex.before?(Timex.today, Timex.shift(Timex.today, days: 1)) true &gt; Timex.before?(Timex.shift(Timex.today, days: 1), Timex.today) false &gt; interval = Timex.Interval.new(from: ~D[2016-03-03], until: [days: 3]) %Timex.Interval{from: ~N[2016-03-03 00:00:00], left_open: false, right_open: true, step: [days: 1], until: ~N[2016-03-06 00:00:00]} &gt; ~D[2016-03-04] in interval true &gt; ~N[2016-03-04 00:00:00] in interval true &gt; ~N[2016-03-02 00:00:00] in interval false &gt; Timex.Interval.overlaps?(Timex.Interval.new(from: ~D[2016-03-04], until: [days: 1]), interval) true &gt; Timex.Interval.overlaps?(Timex.Interval.new(from: ~D[2016-03-07], until: [days: 1]), interval) false There are a ton of other functions, all of which work with Erlang datetime tuples, Date, NaiveDateTime, and DateTime. The Duration module contains functions for working with Durations, including Erlang timestamps (such as those returned from :timer.tc)ExtensibilityTimex exposes a number of extension points for you, in order to accommodate different use cases:Timex itself defines it's core operations on the Date, DateTime, and NaiveDateTime types using the Timex.Protocol protocol. From there, all other Timex functionality is derived. If you have custom date/datetime types you want to use with Timex, this is the protocol you would need to implement.Timex also defines a Timex.Comparable protocol, which you can extend to add comparisons to custom date/datetime types.You can provide your own formatter/parser for datetime strings by implementing the Timex.Format.DateTime.Formatter and/or Timex.Parse.DateTime.Parser behaviours, depending on your needs.Common Issuesiso_* FunctionsTimex functions of the form iso_* behave based on how the ISO calendar represents dates/times and not the ISO8601 date format. This confusion has occurred before, and it's important to note this!Timex with escriptIf you need to use Timex from within an escript, add {:tzdata, &quot;~&gt; 0.1.8&quot;, override: true} to your deps, more recent versions of :tzdata are unable to work in an escript because of the need to load ETS table files from priv, and due to the way ETS loads these files, it's not possible to do so.If your build still throws an error after this, try removing the _build and deps folder. Then execute mix deps.unlock tzdata and mix deps.get.Automatic time zone updatesTimex includes the Tzdata library for time zone data. Tzdata has an automatic update capability that fetches updates from IANA and which is enabled by default; if you want to disable it, check the Tzdata documentation for details.MigratingIf you have been using Timex pre-3.x, and you are looking to migrate, it will be fairly painless as long as you review the CHANGELOG.md file and make note of anything that has changed which you are using. In almost every single case, the functionality has simply moved, or is accessed slightly differently. In some cases behaviour has changed, but the old behaviour can be achieved manually. For those things which were removed or are no longer available, it is almost certainly because those things are no longer recommended, or no longer make sense for this library. If I have missed something, or there is a good justification for re-adding something which was removed, I'm always open to discussing it on the issue tracker.Overview of 3.x changesPlease see the CHANGELOG.md file for the list of all changes made, below are a brief recap of the major points, and instructions on how to migrate your existing Timex-based code to 3.x.Virtually all of Timex's API is consolidated around three modules: Timex, Duration, and Interval. There are public APIs in other modules of course, but most everything you do can be done via one of those three.All Timex-provided types have been removed, with the exception of Time which has been renamed to Duration to better fit it's purpose, TimezoneInfo, which is still used internally, and accessible externally, and AmbiguousDateTime, which is still required when dealing with ambiguous DateTime structs. Date and DateTime are now part of Elixir 1.3, and NaiveDateTime has been added as a first class type as well.Conversions have been rehashed, and are now accessed via the Timex module: to_date, to_datetime, to_naive_datetime, to_erl, to_unix, to_gregorian_seconds, and to_gregorian_microseconds. Use these to convert back and forth between types. With very few exceptions, there are no from_* conversions. You must construct a standard type either with the standard library functions (such as NaiveDateTime.new), or by converting from another standard type (i.e. Erlang datetime tuple to NaiveDateTime).A variety of improvements came about as well:Microsecond precision everywhere it can be maintainedVarious bug fixes from the 2.x release which were held backA lot of code clean upFaster compilationFixed typespecs, docs, and tests across the boardAdded Julian calendar supportRoadmapThe following are an unordered list of things I plan for Timex in the future, if you have specific requests, please open an issue with &quot;RFC&quot; in the title, and we can discuss it, and hopefully get input from the community.100% test coverage (well under way!)QuickCheck tests (haven't started this, but I really want to start ASAP){ASP.NET} formatting/parsing token for interop with .NET services (probably in the next release)Relative time formatter/parser, along the lines of Moment.js's fromNow, toNow, timeTo, and timeFrom formatting functions.Calendar time formatter/parser, along the lines of Moment.js's calendar time formatterRecurring dates/times APILicenseThis software is licensed under the MIT license.","ref":"Timex.html","title":"Timex","type":"module"},{"doc":"Add time to a date using a Duration Same as shift(date, Duration.from_minutes(5), :duration).","ref":"Timex.html#add/2","title":"Timex.add/2","type":"function"},{"doc":"Returns a boolean indicating whether the first Timex.Comparable occurs after the second","ref":"Timex.html#after?/2","title":"Timex.after?/2","type":"function"},{"doc":"Returns a boolean indicating whether the first Timex.Comparable occurs before the second","ref":"Timex.html#before?/2","title":"Timex.before?/2","type":"function"},{"doc":"Returns a DateTime representing the beginning of the dayExamplesiex&gt; date = Timex.to_datetime({{2015, 1, 1}, {13, 14, 15}}, &quot;Etc/UTC&quot;) iex&gt; Timex.beginning_of_day(date) Timex.to_datetime({{2015, 1, 1}, {0, 0, 0}}, &quot;Etc/UTC&quot;) iex&gt; date = ~D[2015-01-01] ...&gt; Timex.beginning_of_day(date) ~D[2015-01-01]","ref":"Timex.html#beginning_of_day/1","title":"Timex.beginning_of_day/1","type":"function"},{"doc":"Given a date returns a date at the beginning of the month.iex&gt; date = Timex.to_datetime({{2015, 6, 15}, {12,30,0}}, &quot;Europe/Paris&quot;) iex&gt; Timex.beginning_of_month(date) Timex.to_datetime({{2015, 6, 1}, {0, 0, 0}}, &quot;Europe/Paris&quot;)","ref":"Timex.html#beginning_of_month/1","title":"Timex.beginning_of_month/1","type":"function"},{"doc":"Same as beginning_of_month/1, except takes year and month as distinct arguments","ref":"Timex.html#beginning_of_month/2","title":"Timex.beginning_of_month/2","type":"function"},{"doc":"Given a date returns a date at the beginning of the quarter.iex&gt; date = Timex.to_datetime({{2015, 6, 15}, {12,30,0}}, &quot;America/Chicago&quot;) iex&gt; Timex.beginning_of_quarter(date) Timex.to_datetime({{2015, 4, 1}, {0, 0, 0}}, &quot;America/Chicago&quot;)","ref":"Timex.html#beginning_of_quarter/1","title":"Timex.beginning_of_quarter/1","type":"function"},{"doc":"Shifts the date to the beginning of the weekThe weekstart can between 1..7, an atom e.g. :mon, or a string e.g. &quot;Monday&quot;Examplesiex&gt; date = ~N[2015-11-30T13:30:30] # Monday 30th November iex&gt; Timex.beginning_of_week(date) ~N[2015-11-30T00:00:00] iex&gt; date = ~D[2015-11-30] # Monday 30th November iex&gt; Timex.beginning_of_week(date, :sun) ~D[2015-11-29]","ref":"Timex.html#beginning_of_week/2","title":"Timex.beginning_of_week/2","type":"function"},{"doc":"Given a date or a number create a date at the beginning of that yearExamplesiex&gt; date = ~N[2015-06-15T00:00:00] iex&gt; Timex.beginning_of_year(date) ~N[2015-01-01T00:00:00] iex&gt; Timex.beginning_of_year(2015) ~D[2015-01-01]","ref":"Timex.html#beginning_of_year/1","title":"Timex.beginning_of_year/1","type":"function"},{"doc":"Returns a boolean indicating whether the first Timex.Comparable occurs between the second and third.If an error occurs, an error tuple will be returned.By default, the start and end bounds are exclusive. You can opt for inclusive bounds with the inclusive: true option. To set just one of the bounds as inclusive, use the inclusive: :start or inclusive: :end option.","ref":"Timex.html#between?/4","title":"Timex.between?/4","type":"function"},{"doc":"Gets the current centuryExamplesiex&gt; Elixir.Timex.century 21","ref":"Timex.html#century/0","title":"Timex.century/0","type":"function"},{"doc":"Given a date, get the century this date is in.Examplesiex&gt; Timex.today |&gt; Elixir.Timex.century 21 iex&gt; Timex.now |&gt; Elixir.Timex.century 21 iex&gt; Elixir.Timex.century(2016) 21","ref":"Timex.html#century/1","title":"Timex.century/1","type":"function"},{"doc":"See docs for compare/3","ref":"Timex.html#compare/2","title":"Timex.compare/2","type":"function"},{"doc":"Compare two Timex.Comparable values, returning one of the following values:-1 -- the first date comes before the second one0 -- both arguments represent the same date when coalesced to the same timezone.1 -- the first date comes after the second oneYou can provide a few reference constants for the second argument::epoch will compare the first parameter against the Date/DateTime of the first moment of the UNIX epoch:zero will compare the first parameter against the Date/DateTime of the first moment of year zero:distant_past will compare the first parameter against a date/time infinitely in the past (i.e. it will always return 1):distant_future will compare the first parameter against a date/time infinitely in the future (i.e. it will always return -1)You can optionally specify a comparison granularity, any of the following::year:years:month:months:week:weeks:calendar_week (weeks of the calendar as opposed to actual weeks in terms of days):calendar_weeks:day:days:hour:hours:minute:minutes:second:seconds:millisecond:milliseconds:microsecond (default):microseconds:durationand the dates will be compared with the corresponding accuracy. The default granularity is :microsecond.Examplesiex&gt; date1 = ~D[2014-03-04] iex&gt; date2 = ~D[2015-03-04] iex&gt; Timex.compare(date1, date2, :year) -1 iex&gt; Timex.compare(date2, date1, :year) 1 iex&gt; Timex.compare(date1, date1) 0","ref":"Timex.html#compare/3","title":"Timex.compare/3","type":"function"},{"doc":"Returns the ordinal day number of the date.Examplesiex&gt; Timex.day(~D[2015-06-26]) 177","ref":"Timex.html#day/1","title":"Timex.day/1","type":"function"},{"doc":"Get the name of the day corresponding to the provided numberExamplesiex&gt; Elixir.Timex.day_name(1) &quot;Monday&quot; iex&gt; Elixir.Timex.day_name(0) {:error, :invalid_weekday_number}","ref":"Timex.html#day_name/1","title":"Timex.day_name/1","type":"function"},{"doc":"Get the short name of the day corresponding to the provided numberExamplesiex&gt; Elixir.Timex.day_shortname(1) &quot;Mon&quot; iex&gt; Elixir.Timex.day_shortname(0) {:error, :invalid_weekday_number}","ref":"Timex.html#day_shortname/1","title":"Timex.day_shortname/1","type":"function"},{"doc":"Get the day of the week corresponding to the given name.The name can be given as a string of the weekday name or its first three characters (lowercase or capitalized) or as a corresponding atom (lowercase only).Examplesiex&gt; Elixir.Timex.day_to_num(&quot;Monday&quot;) 1 iex&gt; Elixir.Timex.day_to_num(&quot;monday&quot;) 1 iex&gt; Elixir.Timex.day_to_num(&quot;Mon&quot;) 1 iex&gt; Elixir.Timex.day_to_num(&quot;mon&quot;) 1 iex&gt; Elixir.Timex.day_to_num(:mon) 1 iex&gt; Elixir.Timex.day_to_num(:sunday) 7","ref":"Timex.html#day_to_num/1","title":"Timex.day_to_num/1","type":"function"},{"doc":"Return the number of days in the month which the date falls on.Examplesiex&gt; Timex.days_in_month(~D[1970-01-01]) 31","ref":"Timex.html#days_in_month/1","title":"Timex.days_in_month/1","type":"function"},{"doc":"Same as days_in_month/2, except takes year and month as distinct arguments","ref":"Timex.html#days_in_month/2","title":"Timex.days_in_month/2","type":"function"},{"doc":"Number of days to the beginning of the weekThe weekstart determines which is the first day of the week, defaults to monday. It can be a number between 1..7 (1 is monday, 7 is sunday), or any value accepted by day_to_num/1.Examplesiex&gt; date = ~D[2015-11-30] # Monday 30th November iex&gt; Timex.days_to_beginning_of_week(date) 0 iex&gt; date = ~D[2015-11-30] # Monday 30th November iex&gt; Timex.days_to_beginning_of_week(date, :sun) 1","ref":"Timex.html#days_to_beginning_of_week/2","title":"Timex.days_to_beginning_of_week/2","type":"function"},{"doc":"Number of days to the end of the week.The weekstart can between 1..7, an atom e.g. :mon, or a string e.g. &quot;Monday&quot;ExamplesWeek starting Monday iex&gt; date = ~D[2015-11-30] # Monday 30th November iex&gt; Timex.days_to_end_of_week(date) 6 Week starting Sunday iex&gt; date = ~D[2015-11-30] # Monday 30th November iex&gt; Timex.days_to_end_of_week(date, :sun) 5","ref":"Timex.html#days_to_end_of_week/2","title":"Timex.days_to_end_of_week/2","type":"function"},{"doc":"See docs for diff/3","ref":"Timex.html#diff/2","title":"Timex.diff/2","type":"function"},{"doc":"Calculate time interval between two dates. The result will be a signed integer, negative if the first date/time comes before the second, and positive if the first date/time comes after the second.You must specify one of the following units::year:years:month:months:week:weeks:calendar_week (weeks of the calendar as opposed to actual weeks in terms of days):calendar_weeks:day:days:hour:hours:minute:minutes:second:seconds:millisecond:milliseconds:microsecond (default):microseconds:durationand the result will be an integer value of those units or a Duration.","ref":"Timex.html#diff/3","title":"Timex.diff/3","type":"function"},{"doc":"Returns a DateTime representing the end of the dayExamplesiex&gt; date = ~N[2015-01-01T13:14:15] ...&gt; Timex.end_of_day(date) ~N[2015-01-01T23:59:59]","ref":"Timex.html#end_of_day/1","title":"Timex.end_of_day/1","type":"function"},{"doc":"Given a date returns a date at the end of the month.iex&gt; date = ~N[2015-06-15T12:30:00Z] iex&gt; Timex.end_of_month(date) ~N[2015-06-30T23:59:59Z]","ref":"Timex.html#end_of_month/1","title":"Timex.end_of_month/1","type":"function"},{"doc":"Same as end_of_month/1, except takes year and month as distinct argumentsExamplesiex&gt; Timex.end_of_month(2016, 2) ~D[2016-02-29]","ref":"Timex.html#end_of_month/2","title":"Timex.end_of_month/2","type":"function"},{"doc":"Given a date or a year and month returns a date at the end of the quarter.iex&gt; date = ~N[2015-06-15T12:30:00] ...&gt; Timex.end_of_quarter(date) ~N[2015-06-30T23:59:59] iex&gt; Timex.end_of_quarter(2015, 4) ~D[2015-06-30]","ref":"Timex.html#end_of_quarter/1","title":"Timex.end_of_quarter/1","type":"function"},{"doc":"Same as end_of_quarter/1, except takes year and month as distinct arguments","ref":"Timex.html#end_of_quarter/2","title":"Timex.end_of_quarter/2","type":"function"},{"doc":"Returns a Date or a DateTime representing the end of the week, depending on the input, i.e. if you pass a date/time value which represents just a date, you will get back a Date, if both a date and time are present, you will get back a DateTimeThe weekstart can between 1..7, an atom e.g. :mon, or a string e.g. &quot;Monday&quot;Examplesiex&gt; date = ~N[2015-11-30T13:30:30] # Monday 30th November ...&gt; Timex.end_of_week(date) ~N[2015-12-06T23:59:59] iex&gt; date = ~D[2015-11-30] # Monday 30th November ...&gt; Timex.end_of_week(date, :sun) ~D[2015-12-05]","ref":"Timex.html#end_of_week/2","title":"Timex.end_of_week/2","type":"function"},{"doc":"Given a date or a number create a date at the end of that yearExamplesiex&gt; date = ~N[2015-06-15T00:00:00] iex&gt; Timex.end_of_year(date) ~N[2015-12-31T23:59:59] iex&gt; Timex.end_of_year(2015) ~D[2015-12-31]","ref":"Timex.html#end_of_year/1","title":"Timex.end_of_year/1","type":"function"},{"doc":"Returns a Date representing the start of the UNIX epoch","ref":"Timex.html#epoch/0","title":"Timex.epoch/0","type":"function"},{"doc":"Returns a boolean indicating whether the two Timex.Comparable values are equivalent.Equality here implies that the two Comparables represent the same moment in time (with the given granularity), not equality of the data structure.The options for granularity is the same as for compare/3, defaults to :seconds.Examplesiex&gt; date1 = ~D[2014-03-01] ...&gt; date2 = ~D[2014-03-01] ...&gt; Elixir.Timex.equal?(date1, date2) true iex&gt; date1 = ~D[2014-03-01] ...&gt; date2 = Timex.to_datetime({2014, 3, 1}, &quot;Etc/UTC&quot;) ...&gt; Elixir.Timex.equal?(date1, date2) true","ref":"Timex.html#equal?/3","title":"Timex.equal?/3","type":"function"},{"doc":"Formats a date/time value using the given format string (and optional formatter).See Timex.Format.DateTime.Formatters.Default or Timex.Format.DateTime.Formatters.Strftime for documentation on the syntax supported by those formatters.To use the Default formatter, simply call format/2. To use the Strftime formatter, you can either alias and pass Strftime by module name, or as a shortcut, you can pass :strftime instead.Formatting will convert other dates than Elixir date types (Date, DateTime, NaiveDateTime) to a NaiveDateTime using to_naive_datetime/1 before formatting.Examplesiex&gt; date = ~D[2016-02-29] ...&gt; Timex.format(date, &quot;{YYYY}-{0M}-{D}&quot;) {:ok, &quot;2016-02-29&quot;} iex&gt; datetime = Timex.to_datetime({{2016,2,29},{22,25,0}}, &quot;Etc/UTC&quot;) ...&gt; Timex.format(datetime, &quot;{ISO:Extended}&quot;) {:ok, &quot;2016-02-29T22:25:00+00:00&quot;}","ref":"Timex.html#format/2","title":"Timex.format/2","type":"function"},{"doc":"Same as format/2, except using a custom formatterExamplesiex&gt; use Timex ...&gt; datetime = Timex.to_datetime({{2016,2,29},{22,25,0}}, &quot;America/Chicago&quot;) iex&gt; Timex.format(datetime, &quot;%FT%T%:z&quot;, :strftime) {:ok, &quot;2016-02-29T22:25:00-06:00&quot;}","ref":"Timex.html#format/3","title":"Timex.format/3","type":"function"},{"doc":"Same as format/2, except it returns only the value (not a tuple) and raises on error.Examplesiex&gt; date = ~D[2016-02-29] ...&gt; Timex.format!(date, &quot;{YYYY}-{0M}-{D}&quot;) &quot;2016-02-29&quot;","ref":"Timex.html#format!/2","title":"Timex.format!/2","type":"function"},{"doc":"Same as format/3, except it returns only the value (not a tuple) and raises on error.Examplesiex&gt; use Timex ...&gt; datetime = Timex.to_datetime({{2016,2,29},{22,25,0}}, &quot;America/Chicago&quot;) iex&gt; Timex.format!(datetime, &quot;%FT%T%:z&quot;, :strftime) &quot;2016-02-29T22:25:00-06:00&quot;","ref":"Timex.html#format!/3","title":"Timex.format!/3","type":"function"},{"doc":"Formats an Erlang timestamp using the ISO-8601 duration format, or optionally, with a custom formatter of your choosing.See Timex.Format.Duration.Formatters.Default or Timex.Format.Duration.Formatters.Humanized for documentation on the specific formatter behaviour.To use the Default formatter, simply call format_duration/2. To use the Humanized formatter, you can either alias and pass Humanized by module name, or as a shortcut, you can pass :humanized instead.Examplesiex&gt; use Timex ...&gt; duration = Duration.from_seconds(Timex.to_unix({2016, 2, 29})) ...&gt; Timex.format_duration(duration) &quot;P46Y2M10D&quot; iex&gt; use Timex ...&gt; duration = Duration.from_seconds(Timex.to_unix({2016, 2, 29})) ...&gt; Timex.format_duration(duration, :humanized) &quot;46 years, 2 months, 1 week, 3 days&quot; iex&gt; use Timex ...&gt; datetime = Duration.from_seconds(Timex.to_unix(~N[2016-02-29T22:25:00])) ...&gt; Timex.format_duration(datetime, :humanized) &quot;46 years, 2 months, 1 week, 3 days, 22 hours, 25 minutes&quot;","ref":"Timex.html#format_duration/1","title":"Timex.format_duration/1","type":"function"},{"doc":"Same as format_duration/1, except it also accepts a formatter","ref":"Timex.html#format_duration/2","title":"Timex.format_duration/2","type":"function"},{"doc":"Convert an iso ordinal day number to the day it represents in the current year. ## Examplesiex&gt; %Date{:year =&gt; year} = Timex.from_iso_day(180) ...&gt; %Date{:year =&gt; todays_year} = Timex.today() ...&gt; year == todays_year true","ref":"Timex.html#from_iso_day/1","title":"Timex.from_iso_day/1","type":"function"},{"doc":"Same as from_iso_day/1, except you can expect the following based on the second parameter:If an integer year is given, the result will be a Date structFor any date/time value, the result will be in the same format (i.e. Date -&gt; Date)In all cases, the resulting value will be the date representation of the provided ISO day in that yearExamplesCreating a Date from the given dayiex&gt; use Timex ...&gt; expected = ~D[2015-06-29] ...&gt; (expected === Timex.from_iso_day(180, 2015)) trueCreating a Date/DateTime from the given dayiex&gt; use Timex ...&gt; expected = Timex.to_datetime({{2015, 6, 29}, {0,0,0}}, &quot;Etc/UTC&quot;) ...&gt; beginning = Timex.to_datetime({{2015,1,1}, {0,0,0}}, &quot;Etc/UTC&quot;) ...&gt; (expected === Timex.from_iso_day(180, beginning)) trueShifting a Date/DateTime to the given dayiex&gt; use Timex ...&gt; date = Timex.to_datetime({{2015,6,26}, {12,0,0}}, &quot;Etc/UTC&quot;) ...&gt; expected = Timex.to_datetime({{2015, 6, 29}, {12,0,0}}, &quot;Etc/UTC&quot;) ...&gt; (Timex.from_iso_day(180, date) === expected) true","ref":"Timex.html#from_iso_day/2","title":"Timex.from_iso_day/2","type":"function"},{"doc":"Given an ISO triplet {year, week number, weekday}, convert it to a Date struct.Examplesiex&gt; expected = Timex.to_date({2014, 1, 28}) iex&gt; Timex.from_iso_triplet({2014, 5, 2}) === expected true","ref":"Timex.html#from_iso_triplet/1","title":"Timex.from_iso_triplet/1","type":"function"},{"doc":"Formats a DateTime using a fuzzy relative duration, from now.Examplesiex&gt; use Timex ...&gt; Timex.from_now(Timex.shift(DateTime.utc_now(), days: 2, hours: 1)) &quot;in 2 days&quot; iex&gt; use Timex ...&gt; Timex.from_now(Timex.shift(DateTime.utc_now(), days: -2)) &quot;2 days ago&quot;","ref":"Timex.html#from_now/1","title":"Timex.from_now/1","type":"function"},{"doc":"Formats a DateTime using a fuzzy relative duration, translated using given localeExamplesiex&gt; use Timex ...&gt; Timex.from_now(Timex.shift(DateTime.utc_now(), days: 2, hours: 1), &quot;ru&quot;) &quot;через 2 дня&quot; iex&gt; use Timex ...&gt; Timex.from_now(Timex.shift(DateTime.utc_now(), days: -2), &quot;ru&quot;) &quot;2 дня назад&quot;","ref":"Timex.html#from_now/2","title":"Timex.from_now/2","type":"function"},{"doc":"Formats a DateTime using a fuzzy relative duration, with a reference datetime other than now, translated using the given locale","ref":"Timex.html#from_now/3","title":"Timex.from_now/3","type":"function"},{"doc":"Delegates to DateTime.from_unix!/2. To recap the docs:Converts the given Unix time to DateTime.The integer can be given in different units according to System.convert_time_unit/3 and it will be converted to microseconds internally. Defaults to :second.Unix times are always in UTC and therefore the DateTime will be returned in UTC.","ref":"Timex.html#from_unix/2","title":"Timex.from_unix/2","type":"function"},{"doc":"Return a boolean indicating whether the given year is a leap year. You may pase a date or a year number.Examplesiex&gt; Timex.epoch() |&gt; Elixir.Timex.is_leap? false iex&gt; Elixir.Timex.is_leap?(2012) true","ref":"Timex.html#is_leap?/1","title":"Timex.is_leap?/1","type":"function"},{"doc":"Return a boolean indicating whether the given date is valid.Examplesiex&gt; use Timex ...&gt; Timex.is_valid?(~N[0001-01-01T01:01:01]) true iex&gt; use Timex ...&gt; %Date{year: 1, day: 1, month: 13} |&gt; Elixir.Timex.is_valid? false","ref":"Timex.html#is_valid?/1","title":"Timex.is_valid?/1","type":"function"},{"doc":"Returns a boolean indicating whether the provided term represents a valid time, valid times are one of:{hour, min, sec}{hour, min, sec, ms}","ref":"Timex.html#is_valid_time?/1","title":"Timex.is_valid_time?/1","type":"function"},{"doc":"Returns a boolean indicating whether the provided term represents a valid timezone, valid timezones are one of:TimezoneInfo structA timezone name as a string:utc as a shortcut for the UTC timezone:local as a shortcut for the local timezoneA number representing an offset from UTC","ref":"Timex.html#is_valid_timezone?/1","title":"Timex.is_valid_timezone?/1","type":"function"},{"doc":"Return a 3-tuple {year, week number, weekday} for the given Date/DateTime.Examplesiex&gt; Elixir.Timex.iso_triplet(Timex.epoch) {1970, 1, 4}","ref":"Timex.html#iso_triplet/1","title":"Timex.iso_triplet/1","type":"function"},{"doc":"Return a pair {year, week number} (as defined by ISO 8601) that the given Date/DateTime value falls on.Examplesiex&gt; Elixir.Timex.iso_week({1970, 1, 1}) {1970,1}","ref":"Timex.html#iso_week/1","title":"Timex.iso_week/1","type":"function"},{"doc":"Same as iso_week/1, except this takes a year, month, and day as distinct arguments.Examplesiex&gt; Elixir.Timex.iso_week(1970, 1, 1) {1970,1}","ref":"Timex.html#iso_week/3","title":"Timex.iso_week/3","type":"function"},{"doc":"Same as format/2, except takes a locale name to translate text to.Translations only apply to units, relative time phrases, and only for the locales in the list of supported locales in the Timex documentation.","ref":"Timex.html#lformat/3","title":"Timex.lformat/3","type":"function"},{"doc":"Same as lformat/3, except takes a formatter as it's last argument.Translations only apply to units, relative time phrases, and only for the locales in the list of supported locales in the Timex documentation.","ref":"Timex.html#lformat/4","title":"Timex.lformat/4","type":"function"},{"doc":"Same as lformat/3, except local_format! raises on error.See lformat/3 docs for usage examples.","ref":"Timex.html#lformat!/3","title":"Timex.lformat!/3","type":"function"},{"doc":"Same as lformat/4, except local_format! raises on error.See lformat/4 docs for usage examples","ref":"Timex.html#lformat!/4","title":"Timex.lformat!/4","type":"function"},{"doc":"Same as format_duration/1, except takes a locale for use in translation","ref":"Timex.html#lformat_duration/2","title":"Timex.lformat_duration/2","type":"function"},{"doc":"Same as lformat_duration/2, except takes a formatter as an argument","ref":"Timex.html#lformat_duration/3","title":"Timex.lformat_duration/3","type":"function"},{"doc":"Returns a DateTime representing the current moment in time in the local timezone.Exampleiex&gt; %DateTime{time_zone: tz} = Timex.local(); ...&gt; tz != nil true","ref":"Timex.html#local/0","title":"Timex.local/0","type":"function"},{"doc":"Returns a DateTime representing the given date/time in the local timezoneExampleiex&gt; %DateTime{time_zone: tz} = Timex.local(DateTime.utc_now()); ...&gt; tz != nil true","ref":"Timex.html#local/1","title":"Timex.local/1","type":"function"},{"doc":"Get the name of the month corresponding to the provided numberExamplesiex&gt; Elixir.Timex.month_name(1) &quot;January&quot; iex&gt; Elixir.Timex.month_name(0) {:error, :invalid_month_number}","ref":"Timex.html#month_name/1","title":"Timex.month_name/1","type":"function"},{"doc":"Get the short name of the month corresponding to the provided numberExamplesiex&gt; Elixir.Timex.month_shortname(1) &quot;Jan&quot; iex&gt; Elixir.Timex.month_shortname(0) {:error, :invalid_month_number}","ref":"Timex.html#month_shortname/1","title":"Timex.month_shortname/1","type":"function"},{"doc":"Get the number of the month corresponding to the given name.Examplesiex&gt; Elixir.Timex.month_to_num(&quot;January&quot;) 1 iex&gt; Elixir.Timex.month_to_num(&quot;january&quot;) 1 iex&gt; Elixir.Timex.month_to_num(&quot;Jan&quot;) 1 iex&gt; Elixir.Timex.month_to_num(&quot;jan&quot;) 1 iex&gt; Elixir.Timex.month_to_num(:jan) 1","ref":"Timex.html#month_to_num/1","title":"Timex.month_to_num/1","type":"function"},{"doc":"Given a unit to normalize, and the value to normalize, produces a valid value for that unit, clamped to whatever boundaries are defined for that unit.Exampleiex&gt; Timex.normalize(:hour, 26) 23","ref":"Timex.html#normalize/2","title":"Timex.normalize/2","type":"function"},{"doc":"Returns a DateTime representing the current moment in time in UTC","ref":"Timex.html#now/0","title":"Timex.now/0","type":"function"},{"doc":"Returns a DateTime representing the current moment in time in the provided timezone.","ref":"Timex.html#now/1","title":"Timex.now/1","type":"function"},{"doc":"Parses a datetime string into a DateTime struct, using the provided format string (and optional tokenizer).See Timex.Format.DateTime.Formatters.Default or Timex.Format.DateTime.Formatters.Strftime for documentation on the syntax supported in format strings by their respective tokenizers.To use the Default tokenizer, simply call parse/2. To use the Strftime tokenizer, you can either alias and pass Timex.Parse.DateTime.Tokenizer.Strftime by module name, or as a shortcut, you can pass :strftime instead.Examplesiex&gt; use Timex ...&gt; {:ok, result} = Timex.parse(&quot;2016-02-29&quot;, &quot;{YYYY}-{0M}-{D}&quot;) ...&gt; result ~N[2016-02-29T00:00:00] iex&gt; use Timex ...&gt; expected = Timex.to_datetime({{2016, 2, 29}, {22, 25, 0}}, &quot;America/Chicago&quot;) ...&gt; {:ok, result} = Timex.parse(&quot;2016-02-29T22:25:00-06:00&quot;, &quot;{ISO:Extended}&quot;) ...&gt; Timex.equal?(expected, result) true iex&gt; use Timex ...&gt; expected = Timex.to_datetime({{2016, 2, 29}, {22, 25, 0}}, &quot;America/Chicago&quot;) ...&gt; {:ok, result} = Timex.parse(&quot;2016-02-29T22:25:00-06:00&quot;, &quot;%FT%T%:z&quot;, :strftime) ...&gt; Timex.equal?(expected, result) true","ref":"Timex.html#parse/2","title":"Timex.parse/2","type":"function"},{"doc":"See Timex.Parse.DateTime.Parser.parse/3.","ref":"Timex.html#parse/3","title":"Timex.parse/3","type":"function"},{"doc":"Same as parse/2 and parse/3, except parse! raises on error.See parse/2 or parse/3 docs for usage examples.","ref":"Timex.html#parse!/2","title":"Timex.parse!/2","type":"function"},{"doc":"See Timex.Parse.DateTime.Parser.parse!/3.","ref":"Timex.html#parse!/3","title":"Timex.parse!/3","type":"function"},{"doc":"Returns what quarter of the year the given date/time falls in.Examplesiex&gt; Timex.quarter(4) 2","ref":"Timex.html#quarter/1","title":"Timex.quarter/1","type":"function"},{"doc":"Return a new date/time value with the specified fields replaced by new values.Values are automatically validated and clamped to good values by default. If you wish to skip validation, perhaps for performance reasons, pass validate: false.Values are applied in order, so if you pass [datetime: dt, date: d], the date value from date will override datetime's date value.Options which do not apply to the input value (for example, :hour against a Date struct), will be ignored.Exampleiex&gt; use Timex ...&gt; expected = ~D[2015-02-28] ...&gt; result = Timex.set(expected, [month: 2, day: 30]) ...&gt; result == expected true iex&gt; use Timex ...&gt; expected = ~N[2016-02-29T23:30:00] ...&gt; result = Timex.set(expected, [hour: 30]) ...&gt; result === expected true","ref":"Timex.html#set/2","title":"Timex.set/2","type":"function"},{"doc":"A single function for adjusting the date using various units: duration, microseconds, seconds, minutes, hours, days, weeks, months, years.The result of applying the shift will be the same type as that of the input, with the exception of shifting DateTimes, which may result in an AmbiguousDateTime if the shift moves to an ambiguous time period for the zone of that DateTime.Shifting by months will always return a date in the expected month. Because months have different number of days, shifting to a month with fewer days may not be the same day of the month as the original date.If an error occurs, an error tuple will be returned.ExamplesShifting across timezone changesiex&gt; use Timex ...&gt; datetime = Timex.to_datetime({{2016,3,13}, {1,0,0}}, &quot;America/Chicago&quot;) ...&gt; # 2-3 AM doesn&#39;t exist due to leap forward, shift accounts for this ...&gt; %DateTime{hour: 3} = Timex.shift(datetime, hours: 1) ...&gt; shifted = Timex.shift(datetime, hours: 1) ...&gt; {datetime.zone_abbr, shifted.zone_abbr, shifted.hour} {&quot;CST&quot;, &quot;CDT&quot;, 3}Shifting and leap daysiex&gt; use Timex ...&gt; date = ~D[2016-02-29] ...&gt; Timex.shift(date, years: -1) ~D[2015-03-01]Shifting by monthsiex&gt; date = ~D[2016-01-15] ...&gt; Timex.shift(date, months: 1) ~D[2016-02-15] iex&gt; date = ~D[2016-01-31] ...&gt; Timex.shift(date, months: 1) ~D[2016-02-29] iex&gt; date = ~D[2016-01-31] ...&gt; Timex.shift(date, months: 2) ~D[2016-03-31] ...&gt; Timex.shift(date, months: 1) |&gt; Timex.shift(months: 1) ~D[2016-03-29]","ref":"Timex.html#shift/2","title":"Timex.shift/2","type":"function"},{"doc":"Subtract time from a date using a Duration Same as shift(date, Duration.from_minutes(5) |&gt; Duration.invert, :timestamp).","ref":"Timex.html#subtract/2","title":"Timex.subtract/2","type":"function"},{"doc":"Get a TimezoneInfo object for the specified offset or name.When offset or name is invalid, exception is raised.If no DateTime value is given for the second parameter, the current date/time will be used (in other words, it will return the current timezone info for the given zone). If one is provided, the timezone info returned will be based on the provided DateTime (or Erlang datetime tuple) value.Examplesiex&gt; date = Timex.to_datetime({2015, 4, 12}) ...&gt; tz = Timex.timezone(:utc, date) ...&gt; tz.full_name &quot;Etc/UTC&quot; iex&gt; tz = Timex.timezone(&quot;America/Chicago&quot;, {2015,4,12}) ...&gt; {tz.full_name, tz.abbreviation} {&quot;America/Chicago&quot;, &quot;CDT&quot;} iex&gt; tz = Elixir.Timex.timezone(+2, {2015, 4, 12}) ...&gt; {tz.full_name, tz.abbreviation} {&quot;Etc/UTC+2&quot;, &quot;+02&quot;}","ref":"Timex.html#timezone/2","title":"Timex.timezone/2","type":"function"},{"doc":"Returns a list of all valid timezone names in the Olson database","ref":"Timex.html#timezones/0","title":"Timex.timezones/0","type":"function"},{"doc":"Convert a date/time value to a Date struct.","ref":"Timex.html#to_date/1","title":"Timex.to_date/1","type":"function"},{"doc":"Convert a date/time value and timezone name to a DateTime struct. If the DateTime is ambiguous and cannot be resolved, an AmbiguousDateTime will be returned, allowing the developer to choose which of the two choices is desired.If no timezone is provided, &quot;Etc/UTC&quot; will be used","ref":"Timex.html#to_datetime/1","title":"Timex.to_datetime/1","type":"function"},{"doc":"See Timex.Protocol.to_datetime/2.","ref":"Timex.html#to_datetime/2","title":"Timex.to_datetime/2","type":"function"},{"doc":"Convert a date/time value to it's Erlang representation","ref":"Timex.html#to_erl/1","title":"Timex.to_erl/1","type":"function"},{"doc":"Convert a date/time value to gregorian microseconds (microseconds since start of year zero)","ref":"Timex.html#to_gregorian_microseconds/1","title":"Timex.to_gregorian_microseconds/1","type":"function"},{"doc":"Convert a date/time value to gregorian seconds (seconds since start of year zero)","ref":"Timex.html#to_gregorian_seconds/1","title":"Timex.to_gregorian_seconds/1","type":"function"},{"doc":"Convert a date/time value to a Julian calendar date number","ref":"Timex.html#to_julian/1","title":"Timex.to_julian/1","type":"function"},{"doc":"Convert a date/time value to a NaiveDateTime struct.","ref":"Timex.html#to_naive_datetime/1","title":"Timex.to_naive_datetime/1","type":"function"},{"doc":"Convert a date/time value to seconds since the UNIX epoch","ref":"Timex.html#to_unix/1","title":"Timex.to_unix/1","type":"function"},{"doc":"Returns a Date representing the current day in UTC","ref":"Timex.html#today/0","title":"Timex.today/0","type":"function"},{"doc":"Returns a Date representing the current day in the provided timezone.","ref":"Timex.html#today/1","title":"Timex.today/1","type":"function"},{"doc":"Given a format string, validates that the format string is valid for the Default formatter.Given a format string and a formatter, validates that the format string is valid for that formatter.Examplesiex&gt; use Timex ...&gt; Timex.validate_format(&quot;{YYYY}-{M}-{D}&quot;) :ok iex&gt; use Timex ...&gt; Timex.validate_format(&quot;{YYYY}-{M}-{V}&quot;) {:error, &quot;Expected end of input at line 1, column 11&quot;} iex&gt; use Timex ...&gt; Timex.validate_format(&quot;%FT%T%:z&quot;, :strftime) :ok","ref":"Timex.html#validate_format/1","title":"Timex.validate_format/1","type":"function"},{"doc":"See Timex.Format.DateTime.Formatter.validate/2.","ref":"Timex.html#validate_format/2","title":"Timex.validate_format/2","type":"function"},{"doc":"Returns the week number of the date provided, starting at 1.Examplesiex&gt; Timex.week_of_month(~D[2016-03-05]) 1 iex&gt; Timex.week_of_month(~N[2016-03-14T00:00:00Z]) 3","ref":"Timex.html#week_of_month/1","title":"Timex.week_of_month/1","type":"function"},{"doc":"Same as week_of_month/1, except takes year, month, and day as distinct argumentsExamplesiex&gt; Timex.week_of_month(2016, 3, 30) 5","ref":"Timex.html#week_of_month/3","title":"Timex.week_of_month/3","type":"function"},{"doc":"Return weekday number (as defined by ISO 8601) of the specified date.Examplesiex&gt; Timex.epoch |&gt; Elixir.Timex.weekday 4 # (i.e. Thursday)","ref":"Timex.html#weekday/1","title":"Timex.weekday/1","type":"function"},{"doc":"Like weekday/1, but accepts a valid starting weekday value.Examplesiex&gt; Timex.epoch() |&gt; Elixir.Timex.weekday(:sunday) 5","ref":"Timex.html#weekday/2","title":"Timex.weekday/2","type":"function"},{"doc":"Provides a version of weekday/1 and weekday/2 that raises on error.","ref":"Timex.html#weekday!/2","title":"Timex.weekday!/2","type":"function"},{"doc":"Returns a Date representing the start of the Gregorian epoch","ref":"Timex.html#zero/0","title":"Timex.zero/0","type":"function"},{"doc":"","ref":"Timex.html#t:between_options/0","title":"Timex.between_options/0","type":"type"},{"doc":"","ref":"Timex.html#t:set_options/0","title":"Timex.set_options/0","type":"type"},{"doc":"","ref":"Timex.html#t:shift_options/0","title":"Timex.shift_options/0","type":"type"},{"doc":"Represents a DateTime which is ambiguous due to timezone rules.Ambiguity #1 - Non-existent timesLet's use American daylight savings time rules as our example here, using America/Chicago as our example. Central Standard Time for that zone ends at 2:00 AM, but Central Daylight Time does not begin until 3:00 AM, this is because at 2:00 AM, our clocks &quot;spring forward&quot; - which is just an easy way of remembering that the offset goes from -6 from UTC, to -5 from UTC. Since there is no timezone period associated with the hours of 2-3 AM in the America/Chicago zone (it's neither CST nor CDT during that hour), one has to decide what the intent is. Timex makes the call that shifting to the next period (i.e. &quot;spring foward&quot; using our example above) makes the most logical sense when working with non-existent time periods.TL;DR - Timex will &quot;spring forward&quot; or &quot;fall back&quot;, depending on what the zone change happens to be for the non-existent time. Using America/Chicago as an example, if you try to create a DateTime for 2 AM on March 13, 2016, Timex will give you back 3 AM on March 13, 2016, because the zone is in the middle of changing from CST to CDT, and the earliest representable time in CDT is 3 AM.Ambiguity #2 - Times with more than one valid zone periodThis one is the reason why this module exists. There are times, though rare, where more than one zone applies to a given date and time. For example, Asia/Taipei, on December 31st, 1895, from 23:54:00 to 23:59:59, two timezone periods are active LMT, and JWST, because that locale was switching to JWST from LMT. Because of this, it's impossible to know programmaticaly which zone is desired. The programmer must make a choice on which zone they want to use.For this use case, Timex will return an AmbiguousDateTime any time you try to create a DateTime, or shift a DateTime, to an ambiguous time period. It has two fields, :before, containing a DateTime configured in the timezone occurring before the ambiguous period, and :after, containing a DateTime configured in the timezone occurring after the ambiguous period. It is up to you as the programmer to decide which DateTime is the one to use, but my recommendation is to choose :after, unless you have a specific reason to use :before.","ref":"Timex.AmbiguousDateTime.html","title":"Timex.AmbiguousDateTime","type":"module"},{"doc":"","ref":"Timex.AmbiguousDateTime.html#t:t/0","title":"Timex.AmbiguousDateTime.t/0","type":"type"},{"doc":"Represents a choice of two possible timezone periods for a given point in time.","ref":"Timex.AmbiguousTimezoneInfo.html","title":"Timex.AmbiguousTimezoneInfo","type":"module"},{"doc":"","ref":"Timex.AmbiguousTimezoneInfo.html#new/2","title":"Timex.AmbiguousTimezoneInfo.new/2","type":"function"},{"doc":"","ref":"Timex.AmbiguousTimezoneInfo.html#t:t/0","title":"Timex.AmbiguousTimezoneInfo.t/0","type":"type"},{"doc":"This module contains functions for working with dates in the Julian calendar.","ref":"Timex.Calendar.Julian.html","title":"Timex.Calendar.Julian","type":"module"},{"doc":"Returns the day of the week, starting with 0 for Sunday, or 1 for Monday","ref":"Timex.Calendar.Julian.html#day_of_week/2","title":"Timex.Calendar.Julian.day_of_week/2","type":"function"},{"doc":"Same as day_of_week/1, except takes year/month/day as distinct arguments","ref":"Timex.Calendar.Julian.html#day_of_week/4","title":"Timex.Calendar.Julian.day_of_week/4","type":"function"},{"doc":"Returns the Julian day number for the given Erlang date (gregorian)The Julian date (JD) is a continuous count of days from 1 January 4713 BC (= -4712 January 1), Greenwich mean noon (= 12h UT). For example, AD 1978 January 1, 0h UT is JD 2443509.5 and AD 1978 July 21, 15h UT, is JD 2443711.125.This algorithm assumes a proleptic Gregorian calendar (i.e. dates back to year 0), unlike the NASA or US Naval Observatory algorithm - however they align perfectly for dates back to October 15th, 1582, which is where it starts to differ, which is due to the fact that their algorithm assumes there is no Gregorian calendar before that date.","ref":"Timex.Calendar.Julian.html#julian_date/1","title":"Timex.Calendar.Julian.julian_date/1","type":"function"},{"doc":"Same as julian_date/1, except takes year/month/day as distinct arguments","ref":"Timex.Calendar.Julian.html#julian_date/3","title":"Timex.Calendar.Julian.julian_date/3","type":"function"},{"doc":"Same as julian_date/1, except takes year/month/day/hour/minute/second as distinct arguments","ref":"Timex.Calendar.Julian.html#julian_date/6","title":"Timex.Calendar.Julian.julian_date/6","type":"function"},{"doc":"This protocol is used for comparing and diffing different date/time representations","ref":"Timex.Comparable.html","title":"Timex.Comparable","type":"protocol"},{"doc":"Compare two date or datetime types.You can optionally specify a comparison granularity, any of the following::year:years:month:months:week:weeks:calendar_week (weeks of the calendar as opposed to actual weeks in terms of days):calendar_weeks:day:days:hour:hours:minute:minutes:second:seconds:millisecond:milliseconds:microsecond (default):microseconds:durationand the dates will be compared with the corresponding accuracy. The default granularity is :microsecond.0: when equal-1: when the first date/time comes before the second1: when the first date/time comes after the second: when there was a problem comparing, perhaps due to a value being passed which is not a valid date/datetimeExamplesiex&gt; use Timex iex&gt; date1 = ~D[2014-03-04] iex&gt; date2 = ~D[2015-03-04] iex&gt; Timex.compare(date1, date2, :year) -1 iex&gt; Timex.compare(date2, date1, :year) 1 iex&gt; Timex.compare(date1, date1) 0","ref":"Timex.Comparable.html#compare/3","title":"Timex.Comparable.compare/3","type":"function"},{"doc":"Get the difference between two date or datetime types.You can optionally specify a diff granularity, any of the following::year:years:month:months:week:weeks:calendar_week (weeks of the calendar as opposed to actual weeks in terms of days):calendar_weeks:day:days:hour:hours:minute:minutes:second:seconds:millisecond:milliseconds:microsecond (default):microseconds:durationand the result will be an integer value of those units or a Duration struct. The diff value will be negative if a comes before b, and positive if a comes after b. This behaviour mirrors compare/3.When using granularity of :months, the number of days in the month varies. This behavior mirrors Timex.shift/2.Examplesiex&gt; use Timex iex&gt; date1 = ~D[2015-01-28] iex&gt; date2 = ~D[2015-02-28] iex&gt; Timex.diff(date1, date2, :month) -1 iex&gt; Timex.diff(date2, date1, :month) 1 iex&gt; use Timex iex&gt; date1 = ~D[2015-01-31] iex&gt; date2 = ~D[2015-02-28] iex&gt; Timex.diff(date1, date2, :month) -1 iex&gt; Timex.diff(date2, date1, :month) 0","ref":"Timex.Comparable.html#diff/3","title":"Timex.Comparable.diff/3","type":"function"},{"doc":"","ref":"Timex.Comparable.html#t:comparable/0","title":"Timex.Comparable.comparable/0","type":"type"},{"doc":"","ref":"Timex.Comparable.html#t:compare_result/0","title":"Timex.Comparable.compare_result/0","type":"type"},{"doc":"","ref":"Timex.Comparable.html#t:constants/0","title":"Timex.Comparable.constants/0","type":"type"},{"doc":"","ref":"Timex.Comparable.html#t:diff_result/0","title":"Timex.Comparable.diff_result/0","type":"type"},{"doc":"","ref":"Timex.Comparable.html#t:granularity/0","title":"Timex.Comparable.granularity/0","type":"type"},{"doc":"","ref":"Timex.Comparable.html#t:t/0","title":"Timex.Comparable.t/0","type":"type"},{"doc":"","ref":"Timex.Date.html","title":"Timex.Date","type":"module"},{"doc":"","ref":"Timex.DateTime.html","title":"Timex.DateTime","type":"module"},{"doc":"This module provides a friendly API for working with Erlang timestamps, i.e. {megasecs, secs, microsecs}. In addition, it provides an easy way to wrap the measurement of function execution time (via measure).","ref":"Timex.Duration.html","title":"Timex.Duration","type":"module"},{"doc":"Returns the absolute value of the provided Duration.Exampleiex&gt; d = %Timex.Duration{megaseconds: -1, seconds: -2, microseconds: -3} ...&gt; Timex.Duration.abs(d) %Timex.Duration{megaseconds: 1, seconds: 2, microseconds: 3}","ref":"Timex.Duration.html#abs/1","title":"Timex.Duration.abs/1","type":"function"},{"doc":"Add one Duration to another.Examplesiex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 1, microseconds: 1} ...&gt; Timex.Duration.add(d, d) %Timex.Duration{megaseconds: 2, seconds: 2, microseconds: 2} iex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 750000, microseconds: 750000} ...&gt; Timex.Duration.add(d, d) %Timex.Duration{megaseconds: 3, seconds: 500001, microseconds: 500000}","ref":"Timex.Duration.html#add/2","title":"Timex.Duration.add/2","type":"function"},{"doc":"This function determines the difference in time between two timestamps (represented by Duration structs). If the second timestamp is omitted, Duration.now will be used as the reference timestamp. If the first timestamp argument occurs before the second, the resulting measurement will be a negative value.The type argument is an atom indicating the units the measurement should be returned in. If no type argument is provided, a Duration will be returned.Valid measurement units for this function are::microseconds, :milliseconds, :seconds, :minutes, :hours, :days, or :weeksExamplesiex&gt; alias Timex.Duration ...&gt; d = Duration.from_erl({1457, 136000, 785000}) ...&gt; Duration.diff(d, Duration.zero, :days) 16865","ref":"Timex.Duration.html#diff/3","title":"Timex.Duration.diff/3","type":"function"},{"doc":"An alias for Duration.diff/3","ref":"Timex.Duration.html#elapsed/3","title":"Timex.Duration.elapsed/3","type":"function"},{"doc":"Returns the duration since the first day of year 0 to Epoch.Exampleiex&gt; Timex.Duration.epoch() %Timex.Duration{megaseconds: 62_167, seconds: 219_200, microseconds: 0}","ref":"Timex.Duration.html#epoch/0","title":"Timex.Duration.epoch/0","type":"function"},{"doc":"Returns the amount of time since the first day of year 0 to Epoch.The argument is an atom indicating the type of time units to return.The allowed unit type atoms are::microseconds:milliseconds:seconds:minutes:hours:days:weeksExamplesiex&gt; Timex.Duration.epoch(:seconds) 62_167_219_200If the specified type is nil, a duration since the first day of year 0 to Epoch is returned.iex&gt; Timex.Duration.epoch(nil) %Timex.Duration{megaseconds: 62_167, seconds: 219_200, microseconds: 0}","ref":"Timex.Duration.html#epoch/1","title":"Timex.Duration.epoch/1","type":"function"},{"doc":"Converts a clock tuple, i.e. {hour, minute, second, microsecond} to a Duration.Exampleiex&gt; Timex.Duration.from_clock({1, 2, 3, 4}) %Timex.Duration{megaseconds: 0, seconds: 3723, microseconds: 4}","ref":"Timex.Duration.html#from_clock/1","title":"Timex.Duration.from_clock/1","type":"function"},{"doc":"Converts an integer value representing days to a Duration","ref":"Timex.Duration.html#from_days/1","title":"Timex.Duration.from_days/1","type":"function"},{"doc":"Converts an Erlang timestamp to a DurationExampleiex&gt; Timex.Duration.from_erl({1, 2, 3}) %Timex.Duration{megaseconds: 1, seconds: 2, microseconds: 3}","ref":"Timex.Duration.html#from_erl/1","title":"Timex.Duration.from_erl/1","type":"function"},{"doc":"Converts an integer value representing hours to a Duration","ref":"Timex.Duration.html#from_hours/1","title":"Timex.Duration.from_hours/1","type":"function"},{"doc":"Converts an integer value representing microseconds to a Duration","ref":"Timex.Duration.html#from_microseconds/1","title":"Timex.Duration.from_microseconds/1","type":"function"},{"doc":"Converts an integer value representing milliseconds to a Duration","ref":"Timex.Duration.html#from_milliseconds/1","title":"Timex.Duration.from_milliseconds/1","type":"function"},{"doc":"Converts an integer value representing minutes to a Duration","ref":"Timex.Duration.html#from_minutes/1","title":"Timex.Duration.from_minutes/1","type":"function"},{"doc":"Converts an integer value representing seconds to a Duration","ref":"Timex.Duration.html#from_seconds/1","title":"Timex.Duration.from_seconds/1","type":"function"},{"doc":"Converts a Time to a DurationExampleiex&gt; Timex.Duration.from_time(~T[01:01:30]) %Timex.Duration{megaseconds: 0, seconds: 3690, microseconds: 0}","ref":"Timex.Duration.html#from_time/1","title":"Timex.Duration.from_time/1","type":"function"},{"doc":"Converts an integer value representing weeks to a Duration","ref":"Timex.Duration.html#from_weeks/1","title":"Timex.Duration.from_weeks/1","type":"function"},{"doc":"Invert a Duration, i.e. a positive duration becomes a negative one, and vice versaExampleiex&gt; d = %Timex.Duration{megaseconds: -1, seconds: -2, microseconds: -3} ...&gt; Timex.Duration.invert(d) %Timex.Duration{megaseconds: 1, seconds: 2, microseconds: 3}","ref":"Timex.Duration.html#invert/1","title":"Timex.Duration.invert/1","type":"function"},{"doc":"Evaluates fun() and measures the elapsed time.Returns {Duration.t, result}.Exampleiex&gt; {_timestamp, result} = Duration.measure(fn -&gt; 2 * 2 end) ...&gt; result == 4 true","ref":"Timex.Duration.html#measure/1","title":"Timex.Duration.measure/1","type":"function"},{"doc":"Evaluates apply(fun, args), and measures execution time.Returns {Duration.t, result}.Exampleiex&gt; {_timestamp, result} = Duration.measure(fn x, y -&gt; x * y end, [2, 4]) ...&gt; result == 8 true","ref":"Timex.Duration.html#measure/2","title":"Timex.Duration.measure/2","type":"function"},{"doc":"Evaluates apply(module, fun, args), and measures execution time.Returns {Duration.t, result}.Exampleiex&gt; {_timestamp, result} = Duration.measure(Enum, :map, [[1,2], &amp;(&amp;1*2)]) ...&gt; result == [2, 4] true","ref":"Timex.Duration.html#measure/3","title":"Timex.Duration.measure/3","type":"function"},{"doc":"","ref":"Timex.Duration.html#normalize/1","title":"Timex.Duration.normalize/1","type":"function"},{"doc":"Returns the amount of time since Epoch.The argument is an atom indicating the type of time units to return.The allowed unit type atoms are::microsecond(s):millisecond(s):second(s):minutes:hours:days:weeksExamplesiex&gt; Timex.Duration.now(:seconds) 1483141644When the argument is omitted or nil, a Duration is returned.iex&gt; Timex.Duration.now %Timex.Duration{megaseconds: 1483, seconds: 141562, microseconds: 536938}","ref":"Timex.Duration.html#now/1","title":"Timex.Duration.now/1","type":"function"},{"doc":"Parses a duration string (in ISO-8601 format) into a Duration struct.","ref":"Timex.Duration.html#parse/1","title":"Timex.Duration.parse/1","type":"function"},{"doc":"Parses a duration string into a Duration struct, using the provided parser module.","ref":"Timex.Duration.html#parse/2","title":"Timex.Duration.parse/2","type":"function"},{"doc":"Same as parse/1, but returns the Duration unwrapped, and raises on error","ref":"Timex.Duration.html#parse!/1","title":"Timex.Duration.parse!/1","type":"function"},{"doc":"Same as parse/2, but returns the Duration unwrapped, and raises on error","ref":"Timex.Duration.html#parse!/2","title":"Timex.Duration.parse!/2","type":"function"},{"doc":"Scale a Duration by some coefficient value, i.e. a scale of 2 is twice is long.Exampleiex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 1, microseconds: 1} ...&gt; Timex.Duration.scale(d, 2) %Timex.Duration{megaseconds: 2, seconds: 2, microseconds: 2}","ref":"Timex.Duration.html#scale/2","title":"Timex.Duration.scale/2","type":"function"},{"doc":"Subtract one Duration from another.Exampleiex&gt; d1 = %Timex.Duration{megaseconds: 3, seconds: 3, microseconds: 3} ...&gt; d2 = %Timex.Duration{megaseconds: 2, seconds: 2, microseconds: 2} ...&gt; Timex.Duration.sub(d1, d2) %Timex.Duration{megaseconds: 1, seconds: 1, microseconds: 1}","ref":"Timex.Duration.html#sub/2","title":"Timex.Duration.sub/2","type":"function"},{"doc":"Converts a Duration to a clock tuple, i.e. {hour,minute,second,microsecond}.Exampleiex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 1, microseconds: 50} ...&gt; Timex.Duration.to_clock(d) {277, 46, 41, 50}","ref":"Timex.Duration.html#to_clock/1","title":"Timex.Duration.to_clock/1","type":"function"},{"doc":"Converts a Duration to its value in daysExampleiex&gt; Duration.to_days(Duration.from_hours(6)) 0.25 iex&gt; Duration.to_days(Duration.from_hours(25), truncate: true) 1","ref":"Timex.Duration.html#to_days/1","title":"Timex.Duration.to_days/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_days/2","title":"Timex.Duration.to_days/2","type":"function"},{"doc":"Converts a Duration to an Erlang timestampExampleiex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 2, microseconds: 3} ...&gt; Timex.Duration.to_erl(d) {1, 2, 3}","ref":"Timex.Duration.html#to_erl/1","title":"Timex.Duration.to_erl/1","type":"function"},{"doc":"Converts a Duration to its value in hoursExampleiex&gt; Duration.to_hours(Duration.from_minutes(105)) 1.75 iex&gt; Duration.to_hours(Duration.from_minutes(105), truncate: true) 1","ref":"Timex.Duration.html#to_hours/1","title":"Timex.Duration.to_hours/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_hours/2","title":"Timex.Duration.to_hours/2","type":"function"},{"doc":"Converts a Duration to its value in microsecondsExampleiex&gt; Duration.to_microseconds(Duration.from_milliseconds(10.5)) 10_500","ref":"Timex.Duration.html#to_microseconds/1","title":"Timex.Duration.to_microseconds/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_microseconds/2","title":"Timex.Duration.to_microseconds/2","type":"function"},{"doc":"Converts a Duration to its value in millisecondsExampleiex&gt; Duration.to_milliseconds(Duration.from_seconds(1)) 1000.0 iex&gt; Duration.to_milliseconds(Duration.from_seconds(1.543)) 1543.0 iex&gt; Duration.to_milliseconds(Duration.from_seconds(1.543), truncate: true) 1543","ref":"Timex.Duration.html#to_milliseconds/1","title":"Timex.Duration.to_milliseconds/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_milliseconds/2","title":"Timex.Duration.to_milliseconds/2","type":"function"},{"doc":"Converts a Duration to its value in minutesExampleiex&gt; Duration.to_minutes(Duration.from_seconds(90)) 1.5 iex&gt; Duration.to_minutes(Duration.from_seconds(65), truncate: true) 1","ref":"Timex.Duration.html#to_minutes/1","title":"Timex.Duration.to_minutes/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_minutes/2","title":"Timex.Duration.to_minutes/2","type":"function"},{"doc":"Converts a Duration to its value in secondsExampleiex&gt; Duration.to_seconds(Duration.from_milliseconds(1500)) 1.5 iex&gt; Duration.to_seconds(Duration.from_milliseconds(1500), truncate: true) 1","ref":"Timex.Duration.html#to_seconds/1","title":"Timex.Duration.to_seconds/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_seconds/2","title":"Timex.Duration.to_seconds/2","type":"function"},{"doc":"Converts a Duration to a string, using the ISO standard for formatting durations.Examplesiex&gt; d = %Timex.Duration{megaseconds: 0, seconds: 3661, microseconds: 0} ...&gt; Timex.Duration.to_string(d) &quot;PT1H1M1S&quot; iex&gt; d = %Timex.Duration{megaseconds: 102, seconds: 656013, microseconds: 33} ...&gt; Timex.Duration.to_string(d) &quot;P3Y3M3DT3H33M33.000033S&quot;","ref":"Timex.Duration.html#to_string/1","title":"Timex.Duration.to_string/1","type":"function"},{"doc":"Converts a Duration to a Time if the duration fits within a 24-hour clock. If it does not, an error tuple is returned.Examplesiex&gt; d = %Timex.Duration{megaseconds: 0, seconds: 4000, microseconds: 0} ...&gt; Timex.Duration.to_time(d) {:ok, ~T[01:06:40]} iex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 0, microseconds: 0} ...&gt; Timex.Duration.to_time(d) {:error, :invalid_time}","ref":"Timex.Duration.html#to_time/1","title":"Timex.Duration.to_time/1","type":"function"},{"doc":"Same as to_time/1, but returns the Time directly. Raises an error if the duration does not fit within a 24-hour clock.Examplesiex&gt; d = %Timex.Duration{megaseconds: 0, seconds: 4000, microseconds: 0} ...&gt; Timex.Duration.to_time!(d) ~T[01:06:40] iex&gt; d = %Timex.Duration{megaseconds: 1, seconds: 0, microseconds: 0} ...&gt; Timex.Duration.to_time!(d) ** (ArgumentError) cannot convert {277, 46, 40} to time, reason: :invalid_time","ref":"Timex.Duration.html#to_time!/1","title":"Timex.Duration.to_time!/1","type":"function"},{"doc":"Converts a Duration to its value in weeksExampleiex&gt; Duration.to_weeks(Duration.from_days(14)) 2.0 iex&gt; Duration.to_weeks(Duration.from_days(13), truncate: true) 1","ref":"Timex.Duration.html#to_weeks/1","title":"Timex.Duration.to_weeks/1","type":"function"},{"doc":"","ref":"Timex.Duration.html#to_weeks/2","title":"Timex.Duration.to_weeks/2","type":"function"},{"doc":"Return a timestamp representing a time lapse of length 0.iex&gt; Timex.Duration.zero |&gt; Timex.Duration.to_seconds 0.0Can be useful for operations on collections of durations. For instance,Enum.reduce(durations, Duration.zero, Duration.add(&amp;1, &amp;2))Can also be used to represent the timestamp of the start of the UNIX epoch, as all Erlang timestamps are relative to this point.","ref":"Timex.Duration.html#zero/0","title":"Timex.Duration.zero/0","type":"function"},{"doc":"","ref":"Timex.Duration.html#t:measurement_units/0","title":"Timex.Duration.measurement_units/0","type":"type"},{"doc":"","ref":"Timex.Duration.html#t:t/0","title":"Timex.Duration.t/0","type":"type"},{"doc":"","ref":"Timex.Duration.html#t:to_options/0","title":"Timex.Duration.to_options/0","type":"type"},{"doc":"","ref":"Timex.Duration.html#t:units/0","title":"Timex.Duration.units/0","type":"type"},{"doc":"This module defines the behaviour for custom DateTime formatters.","ref":"Timex.Format.DateTime.Formatter.html","title":"Timex.Format.DateTime.Formatter","type":"behaviour"},{"doc":"","ref":"Timex.Format.DateTime.Formatter.html#c:format/2","title":"Timex.Format.DateTime.Formatter.format/2","type":"callback"},{"doc":"Formats a Date, DateTime, or NaiveDateTime as a string, using the provided format string and formatter. If a formatter is not provided, the formatter used is Timex.Format.DateTime.Formatters.DefaultFormatter.Formatting will use the configured default locale, &quot;en&quot; if no other default is given.","ref":"Timex.Format.DateTime.Formatter.html#format/3","title":"Timex.Format.DateTime.Formatter.format/3","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatter.html#c:format!/2","title":"Timex.Format.DateTime.Formatter.format!/2","type":"callback"},{"doc":"Formats a Date, DateTime, or NaiveDateTime as a string, using the provided format string and formatter. If a formatter is not provided, the formatter used is Timex.Format.DateTime.Formatters.DefaultFormatter.Formatting will use the configured default locale, &quot;en&quot; if no other default is given.If an error is encountered during formatting, format! will raise.","ref":"Timex.Format.DateTime.Formatter.html#format!/3","title":"Timex.Format.DateTime.Formatter.format!/3","type":"function"},{"doc":"Given a token (as found in Timex.Parsers.Directive), and a Date, DateTime, or NaiveDateTime struct, produce a string representation of the token using values from the struct, using the default locale.","ref":"Timex.Format.DateTime.Formatter.html#format_token/5","title":"Timex.Format.DateTime.Formatter.format_token/5","type":"function"},{"doc":"Given a token (as found in Timex.Parsers.Directive), and a Date, DateTime, or NaiveDateTime struct, produce a string representation of the token using values from the struct.","ref":"Timex.Format.DateTime.Formatter.html#format_token/6","title":"Timex.Format.DateTime.Formatter.format_token/6","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatter.html#c:lformat/3","title":"Timex.Format.DateTime.Formatter.lformat/3","type":"callback"},{"doc":"Formats a Date, DateTime, or NaiveDateTime as a string, using the provided format string, locale, and formatter.If the locale provided does not have translations, &quot;en&quot; is used by default.If a formatter is not provided, the formatter used is Timex.Format.DateTime.Formatters.DefaultFormatter","ref":"Timex.Format.DateTime.Formatter.html#lformat/4","title":"Timex.Format.DateTime.Formatter.lformat/4","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatter.html#c:lformat!/3","title":"Timex.Format.DateTime.Formatter.lformat!/3","type":"callback"},{"doc":"Formats a Date, DateTime, or NaiveDateTime as a string, using the provided format string, locale, and formatter. If the locale does not have translations, &quot;en&quot; will be used by default.If a formatter is not provided, the formatter used is Timex.Format.DateTime.Formatters.DefaultFormatterIf an error is encountered during formatting, lformat! will raise","ref":"Timex.Format.DateTime.Formatter.html#lformat!/4","title":"Timex.Format.DateTime.Formatter.lformat!/4","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatter.html#c:tokenize/1","title":"Timex.Format.DateTime.Formatter.tokenize/1","type":"callback"},{"doc":"Validates the provided format string, using the provided formatter, or if none is provided, the default formatter. Returns :ok when valid, or {:error, reason} if not valid.","ref":"Timex.Format.DateTime.Formatter.html#validate/2","title":"Timex.Format.DateTime.Formatter.validate/2","type":"function"},{"doc":"Date formatting language used by default by the formatting functions in Timex.This is a novel formatting language introduced with DateFormat. Its main advantage is simplicity and usage of mnemonics that are easy to memorize.Directive formatA directive is an optional padding specifier followed by a mnemonic enclosed in braces ({ and }):&lt;padding&gt;{&lt;mnemonic&gt;}Supported padding specifiers:0 -- pads the number with zeros. Applicable to mnemonics that produce numerical result._ -- pads the number with spaces. Applicable to mnemonics that produce numerical result.When padding specifier is omitted, numbers will not be padded.List of all directivesYears and centuries{YYYY} - full year number (0..9999){YY} - the last two digits of the year number (0.99){C} - century number (0..99){WYYYY} - year number (4 digits) corresponding to the date's ISO week (0000..9999){WYY} - year number (2 digits) corresponding to the date's ISO week (00.99)Months{M} - month number (1..12){Mshort} - abbreviated month name (Jan..Dec, no padding){Mfull} - full month name (January..December, no padding)Days and weekdays{D} - day number (1..31){Dord} - ordinal day of the year (1..366){WDmon} - weekday, Monday first (0..6, no padding){WDsun} - weekday, Sunday first (0..6, no padding){WDshort} - abbreviated weekday name (Mon..Sun, no padding){WDfull} - full weekday name (Monday..Sunday, no padding)Weeks{Wiso} - ISO week number (01..53){Wmon} - week number of the year, Monday first (01..53){Wsun} - week number of the year, Sunday first (01..53)Time{h24} - hour of the day (00..23){h12} - hour of the day (1..12){m} - minutes of the hour (00..59){s} - seconds of the minute (00..60){ss} - fractional second, based on precision of microseconds given (.xxx == ms, .xxxxxx == us){s-epoch} - number of seconds since UNIX epoch{am} - lowercase am or pm (no padding){AM} - uppercase AM or PM (no padding)Time zones{Zname} - time zone name, e.g. UTC (no padding){Zabbr} - time zone abbreviation, e.g. CST (no padding){Z} - time zone offset in the form +0230 (no padding){Z:} - time zone offset in the form -07:30 (no padding){Z::} - time zone offset in the form -07:30:00 (no padding)Compound directivesThese are shortcut directives corresponding to parts of the ISO 8601 specification. The benefit of using these over manually constructed ISO formats is that these directives convert the date to UTC for you.{ISO:Basic} - &lt;date&gt;T&lt;time&gt;&lt;offset&gt;.Full date and time specification without separators.{ISO:Basic:Z} - &lt;date&gt;T&lt;time&gt;Z.Full date and time in UTC without separators (e.g. 20070813T134801Z){ISO:Extended} - &lt;date&gt;T&lt;time&gt;&lt;offset&gt;.Full date and time specification with separators. (e.g. 2007-08-13T16:48:01 +03:00){ISO:Extended:Z} - &lt;date&gt;T&lt;time&gt;Z.Full date and time in UTC. (e.g. 2007-08-13T13:48:01Z){ISOdate} - YYYY-MM-DD.That is, 4-digit year number, followed by 2-digit month and day numbers (e.g. 2007-08-13){ISOtime} - hh:mm:ss.That is, 2-digit hour, minute, and second, separated by colons (e.g. 13:04:05). Midnight is 00 hours.{ISOweek} - YYYY-Www.That is, ISO week-based year, followed by ISO week number (e.g. 2007-W09){ISOweek-day} - YYYY-Www-D.That is, an {ISOweek}, additionally followed by weekday (e.g. 2007-W09-1){ISOord} - YYYY-DDD.That is, year number, followed by the ordinal day number (e.g. 2007-113){ASN1:UTCtime} - YYMMDD&lt;time&gt;Z.Full 2-digit year date and time in UTC without separators (e.g. 070813134801Z){ASN1:GeneralizedTime} - YYYYMMDD&lt;time&gt;.Full 4-digit year date and time in local timezone without separators and with optional fractional seconds (e.g. 20070813134801.032){ASN1:GeneralizedTime:Z} - YYYYMMDD&lt;time&gt;Z.Full 4-digit year date and time in UTC without separators and with optional fractional seconds (e.g. 20070813134801.032Z){ASN1:GeneralizedTime:TZ} - YYYYMMDD&lt;time&gt;&lt;offset&gt;.Full 4-digit year date and time in UTC without separators and with optional fractional seconds (e.g. 20070813134801.032-0500)These directives provide support for miscellaneous common formats:{RFC822} - e.g. Mon, 05 Jun 14 23:20:59 UT{RFC822z} - e.g. Mon, 05 Jun 14 23:20:59 Z{RFC1123} - e.g. Tue, 05 Mar 2013 23:25:19 +0200{RFC1123z} - e.g. Tue, 05 Mar 2013 23:25:19 Z{RFC3339} - e.g. 2013-03-05T23:25:19+02:00{RFC3339z} - e.g. 2013-03-05T23:25:19Z{ANSIC} - e.g. Tue Mar 5 23:25:19 2013{UNIX} - e.g. Tue Mar 5 23:25:19 PST 2013{ASN1:UTCtime} - e.g. 130305232519Z{ASN1:GeneralizedTime} - e.g. 20130305232519.928{ASN1:GeneralizedTime:Z} - e.g. 20130305232519.928Z{ASN1:GeneralizedTime:TZ} - e.g. 20130305232519.928-0500{kitchen} - e.g. 3:25PM","ref":"Timex.Format.DateTime.Formatters.Default.html","title":"Timex.Format.DateTime.Formatters.Default","type":"module"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.format/2.","ref":"Timex.Format.DateTime.Formatters.Default.html#format/2","title":"Timex.Format.DateTime.Formatters.Default.format/2","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatters.Default.html#format/3","title":"Timex.Format.DateTime.Formatters.Default.format/3","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.format!/2.","ref":"Timex.Format.DateTime.Formatters.Default.html#format!/2","title":"Timex.Format.DateTime.Formatters.Default.format!/2","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat/3.","ref":"Timex.Format.DateTime.Formatters.Default.html#lformat/3","title":"Timex.Format.DateTime.Formatters.Default.lformat/3","type":"function"},{"doc":"If one wants to use the default formatting semantics with a different tokenizer, this is the way.","ref":"Timex.Format.DateTime.Formatters.Default.html#lformat/4","title":"Timex.Format.DateTime.Formatters.Default.lformat/4","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat!/3.","ref":"Timex.Format.DateTime.Formatters.Default.html#lformat!/3","title":"Timex.Format.DateTime.Formatters.Default.lformat!/3","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.tokenize/1.","ref":"Timex.Format.DateTime.Formatters.Default.html#tokenize/1","title":"Timex.Format.DateTime.Formatters.Default.tokenize/1","type":"function"},{"doc":"Relative time, based on Moment.jsUses localized strings.The format string should contain {relative}, which is where the phrase will be injected.| Range | Sample Output0 secondsnow1 to 45 secondsa few seconds ago45 to 90 secondsa minute ago90 seconds to 45 minutes2 minutes ago ... 45 minutes ago45 to 90 minutesan hour ago90 minutes to 22 hours2 hours ago ... 22 hours ago22 to 36 hoursa day ago36 hours to 25 days2 days ago ... 25 days ago25 to 45 daysa month ago45 to 345 days2 months ago ... 11 months ago345 to 545 days (1.5 years)a year ago546 days+2 years ago ... 20 years ago","ref":"Timex.Format.DateTime.Formatters.Relative.html","title":"Timex.Format.DateTime.Formatters.Relative","type":"module"},{"doc":"Formats a date/time as a relative time formatted stringExamplesiex&gt; Elixir.Timex.Format.DateTime.Formatters.Relative.format(Timex.shift(Timex.now, minutes: -1), &quot;{relative}&quot;) {:ok, &quot;1 minute ago&quot;}","ref":"Timex.Format.DateTime.Formatters.Relative.html#format/2","title":"Timex.Format.DateTime.Formatters.Relative.format/2","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.format!/2.","ref":"Timex.Format.DateTime.Formatters.Relative.html#format!/2","title":"Timex.Format.DateTime.Formatters.Relative.format!/2","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat/3.","ref":"Timex.Format.DateTime.Formatters.Relative.html#lformat/3","title":"Timex.Format.DateTime.Formatters.Relative.lformat/3","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat!/3.","ref":"Timex.Format.DateTime.Formatters.Relative.html#lformat!/3","title":"Timex.Format.DateTime.Formatters.Relative.lformat!/3","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatters.Relative.html#relative_to/3","title":"Timex.Format.DateTime.Formatters.Relative.relative_to/3","type":"function"},{"doc":"","ref":"Timex.Format.DateTime.Formatters.Relative.html#relative_to/4","title":"Timex.Format.DateTime.Formatters.Relative.relative_to/4","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.tokenize/1.","ref":"Timex.Format.DateTime.Formatters.Relative.html#tokenize/1","title":"Timex.Format.DateTime.Formatters.Relative.tokenize/1","type":"function"},{"doc":"Date formatting language defined by the strftime function from the Standard C Library.This implementation in Elixir is mostly compatible with strftime. The exception is the absence of locale-depended results. All directives that imply textual result will produce English names and abbreviations.A complete reference of the directives implemented here is given below.Directive formatA directive is marked by the percent sign (%) followed by one character (&lt;directive&gt;). In addition, a few optional specifiers can be inserted in-between:%&lt;flag&gt;&lt;width&gt;&lt;modifier&gt;&lt;directive&gt;Supported flags:- - don't pad numerical results (overrides default padding if any)0 - use zeros for padding_ - use spaces for padding:, :: - used only in combination with %z; see description of %:z and `%::z` below&lt;width&gt; is a non-negative decimal number specifying the minimum field width.&lt;modifier&gt; can be E or O. These are locale-sensitive modifiers, and as such they are currently ignored by this implementation.List of all directives%% - produces a single % in the outputYears and centuries%Y - full year number (0..9999)%y - the last two digits of the year number (0..99)%C - century number (00..99)%G - year number corresponding to the date's ISO week (0..9999)%g - year number (2 digits) corresponding to the date's ISO week (0..99)Months%m - month number (1..12)%b - abbreviated month name (Jan..Dec, no padding)%h - same is %b%B - full month name (January..December, no padding)Days, and days of week%d - day number (1..31)%e - same as %d, but padded with spaces ( 1..31)%j - ordinal day of the year (001..366)%u - weekday, Monday first (1..7)%w - weekday, Sunday first (0..6)%a - abbreviated weekday name (Mon..Sun, no padding)%A - full weekday name (Monday..Sunday, no padding)Weeks%V - ISO week number (01..53)%W - week number of the year, Monday first (00..52)%U - week number of the year, Sunday first (00..52)Time%H - hour of the day (00..23)%k - same as %H, but padded with spaces ( 0..23)%I - hour of the day (1..12)%l - same as %I, but padded with spaces ( 1..12)%M - minutes of the hour (00..59)%S - seconds of the minute (00..60)%f - microseconds in zero padded decimal form, i.e. 025000%L - milliseconds (000..999)%s - number of seconds since UNIX epoch%P - lowercase am or pm (no padding)%p - uppercase AM or PM (no padding)Time zones%Z - time zone name, e.g. UTC (no padding)%z - time zone offset in the form +0230 (no padding)%:z - time zone offset in the form -07:30 (no padding)%::z - time zone offset in the form -07:30:00 (no padding)Compound directives%D - same as %m/%d/%y%F - same as %Y-%m-%d%R - same as %H:%M%r - same as %I:%M:%S %p%T - same as %H:%M:%S%v - same as %e-%b-%Y","ref":"Timex.Format.DateTime.Formatters.Strftime.html","title":"Timex.Format.DateTime.Formatters.Strftime","type":"module"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.format/2.","ref":"Timex.Format.DateTime.Formatters.Strftime.html#format/2","title":"Timex.Format.DateTime.Formatters.Strftime.format/2","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.format!/2.","ref":"Timex.Format.DateTime.Formatters.Strftime.html#format!/2","title":"Timex.Format.DateTime.Formatters.Strftime.format!/2","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat/3.","ref":"Timex.Format.DateTime.Formatters.Strftime.html#lformat/3","title":"Timex.Format.DateTime.Formatters.Strftime.lformat/3","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.lformat!/3.","ref":"Timex.Format.DateTime.Formatters.Strftime.html#lformat!/3","title":"Timex.Format.DateTime.Formatters.Strftime.lformat!/3","type":"function"},{"doc":"Callback implementation for Timex.Format.DateTime.Formatter.tokenize/1.","ref":"Timex.Format.DateTime.Formatters.Strftime.html#tokenize/1","title":"Timex.Format.DateTime.Formatters.Strftime.tokenize/1","type":"function"},{"doc":"This module defines the behaviour for custom Time formatters","ref":"Timex.Format.Duration.Formatter.html","title":"Timex.Format.Duration.Formatter","type":"behaviour"},{"doc":"Formats a Duration as a string, using the provided formatter. If a formatter is not provided, the formatter used is Timex.Format.Duration.Formatters.Default. As a handy shortcut, you can reference the other built-in formatter (Humanized) via the :humanized atom as shown below.Examplesiex&gt; d = Timex.Duration.from_erl({1435, 180354, 590264}) ...&gt; Elixir.Timex.Format.Duration.Formatter.format(d) &quot;P45Y6M5DT21H12M34.590264S&quot;","ref":"Timex.Format.Duration.Formatter.html#format/1","title":"Timex.Format.Duration.Formatter.format/1","type":"function"},{"doc":"","ref":"Timex.Format.Duration.Formatter.html#c:format/1","title":"Timex.Format.Duration.Formatter.format/1","type":"callback"},{"doc":"Same as format/1, but takes a formatter name as an argumentExamplesiex&gt; d = Timex.Duration.from_erl({1435, 180354, 590264}) ...&gt; Elixir.Timex.Format.Duration.Formatter.format(d, :humanized) &quot;45 years, 6 months, 5 days, 21 hours, 12 minutes, 34 seconds, 590.264 milliseconds&quot;","ref":"Timex.Format.Duration.Formatter.html#format/2","title":"Timex.Format.Duration.Formatter.format/2","type":"function"},{"doc":"Same as format/1, but takes a locale name as an argument, and translates the format string, if the locale has translations.","ref":"Timex.Format.Duration.Formatter.html#lformat/2","title":"Timex.Format.Duration.Formatter.lformat/2","type":"function"},{"doc":"","ref":"Timex.Format.Duration.Formatter.html#c:lformat/2","title":"Timex.Format.Duration.Formatter.lformat/2","type":"callback"},{"doc":"Same as lformat/2, but takes a formatter as an argument","ref":"Timex.Format.Duration.Formatter.html#lformat/3","title":"Timex.Format.Duration.Formatter.lformat/3","type":"function"},{"doc":"Handles formatting Duration values as ISO 8601 durations as described below.Durations are represented by the format P[n]Y[n]M[n]DT[n]H[n]M[n]S. In this representation, the [n] is replaced by the value for each of the date and time elements that follow the [n]. Leading zeros are not required, but the maximum number of digits for each element should be agreed to by the communicating parties. The capital letters P, Y, M, W, D, T, H, M, and S are designators for each of the date and time elements and are not replaced.P is the duration designator (historically called &quot;period&quot;) placed at the start of the duration representation.Y is the year designator that follows the value for the number of years.M is the month designator that follows the value for the number of months.D is the day designator that follows the value for the number of days.T is the time designator that precedes the time components of the representation.H is the hour designator that follows the value for the number of hours.M is the minute designator that follows the value for the number of minutes.S is the second designator that follows the value for the number of seconds.","ref":"Timex.Format.Duration.Formatters.Default.html","title":"Timex.Format.Duration.Formatters.Default","type":"module"},{"doc":"Return a human readable string representing the absolute value of duration (i.e. would return the same output for both negative and positive representations of a given duration)Examplesiex&gt; use Timex ...&gt; Duration.from_erl({0, 1, 1_000_000}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;PT2S&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, 1, 1_000_100}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;PT2.0001S&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, 65, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;PT1M5S&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, -65, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;PT1M5S&quot; iex&gt; use Timex ...&gt; Duration.from_erl({1435, 180354, 590264}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;P45Y6M5DT21H12M34.590264S&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, 0, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Default.format &quot;PT0S&quot;","ref":"Timex.Format.Duration.Formatters.Default.html#format/1","title":"Timex.Format.Duration.Formatters.Default.format/1","type":"function"},{"doc":"Callback implementation for Timex.Format.Duration.Formatter.lformat/2.","ref":"Timex.Format.Duration.Formatters.Default.html#lformat/2","title":"Timex.Format.Duration.Formatters.Default.lformat/2","type":"function"},{"doc":"Handles formatting timestamp values as human readable strings. For formatting timestamps as points in time rather than intervals, use Timex.format","ref":"Timex.Format.Duration.Formatters.Humanized.html","title":"Timex.Format.Duration.Formatters.Humanized","type":"module"},{"doc":"Return a human readable string representing the absolute value of duration (i.e. would return the same output for both negative and positive representations of a given duration)Examplesiex&gt; use Timex ...&gt; Duration.from_erl({0, 1, 1_000_000}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.format &quot;2 seconds&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, 1, 1_000_100}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.format &quot;2 seconds, 100 microseconds&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, 65, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.format &quot;1 minute, 5 seconds&quot; iex&gt; use Timex ...&gt; Duration.from_erl({0, -65, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.format &quot;1 minute, 5 seconds&quot; iex&gt; use Timex ...&gt; Duration.from_erl({1435, 180354, 590264}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.format &quot;45 years, 6 months, 5 days, 21 hours, 12 minutes, 34 seconds, 590.264 milliseconds&quot;","ref":"Timex.Format.Duration.Formatters.Humanized.html#format/1","title":"Timex.Format.Duration.Formatters.Humanized.format/1","type":"function"},{"doc":"Return a human readable string representing the time interval, translated to the given localeExamplesiex&gt; use Timex ...&gt; Duration.from_erl({0, 65, 0}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.lformat(&quot;ru&quot;) &quot;1 минута, 5 секунд&quot; iex&gt; use Timex ...&gt; Duration.from_erl({1435, 180354, 590264}) |&gt; Elixir.Timex.Format.Duration.Formatters.Humanized.lformat(&quot;ru&quot;) &quot;45 лет, 6 месяцев, 5 дней, 21 час, 12 минут, 34 секунды, 590.264 миллисекунд&quot;","ref":"Timex.Format.Duration.Formatters.Humanized.html#lformat/2","title":"Timex.Format.Duration.Formatters.Humanized.lformat/2","type":"function"},{"doc":"","ref":"Timex.Gettext.html","title":"Timex.Gettext","type":"module"},{"doc":"Callback implementation for Gettext.Backend.dgettext/3.","ref":"Timex.Gettext.html#dgettext/3","title":"Timex.Gettext.dgettext/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dgettext_noop/2.","ref":"Timex.Gettext.html#dgettext_noop/2","title":"Timex.Gettext.dgettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dngettext/5.","ref":"Timex.Gettext.html#dngettext/5","title":"Timex.Gettext.dngettext/5","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dngettext_noop/3.","ref":"Timex.Gettext.html#dngettext_noop/3","title":"Timex.Gettext.dngettext_noop/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dpgettext/4.","ref":"Timex.Gettext.html#dpgettext/4","title":"Timex.Gettext.dpgettext/4","type":"macro"},{"doc":"","ref":"Timex.Gettext.html#dpgettext_noop/3","title":"Timex.Gettext.dpgettext_noop/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dpngettext/6.","ref":"Timex.Gettext.html#dpngettext/6","title":"Timex.Gettext.dpngettext/6","type":"macro"},{"doc":"","ref":"Timex.Gettext.html#dpngettext_noop/4","title":"Timex.Gettext.dpngettext_noop/4","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.gettext/2.","ref":"Timex.Gettext.html#gettext/2","title":"Timex.Gettext.gettext/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.gettext_comment/1.","ref":"Timex.Gettext.html#gettext_comment/1","title":"Timex.Gettext.gettext_comment/1","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.gettext_noop/1.","ref":"Timex.Gettext.html#gettext_noop/1","title":"Timex.Gettext.gettext_noop/1","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_bindings/2.","ref":"Timex.Gettext.html#handle_missing_bindings/2","title":"Timex.Gettext.handle_missing_bindings/2","type":"function"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_plural_translation/6.","ref":"Timex.Gettext.html#handle_missing_plural_translation/6","title":"Timex.Gettext.handle_missing_plural_translation/6","type":"function"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_translation/4.","ref":"Timex.Gettext.html#handle_missing_translation/4","title":"Timex.Gettext.handle_missing_translation/4","type":"function"},{"doc":"","ref":"Timex.Gettext.html#lgettext/5","title":"Timex.Gettext.lgettext/5","type":"function"},{"doc":"","ref":"Timex.Gettext.html#lngettext/7","title":"Timex.Gettext.lngettext/7","type":"function"},{"doc":"Callback implementation for Gettext.Backend.ngettext/4.","ref":"Timex.Gettext.html#ngettext/4","title":"Timex.Gettext.ngettext/4","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.ngettext_noop/2.","ref":"Timex.Gettext.html#ngettext_noop/2","title":"Timex.Gettext.ngettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.pgettext/3.","ref":"Timex.Gettext.html#pgettext/3","title":"Timex.Gettext.pgettext/3","type":"macro"},{"doc":"","ref":"Timex.Gettext.html#pgettext_noop/2","title":"Timex.Gettext.pgettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.pngettext/5.","ref":"Timex.Gettext.html#pngettext/5","title":"Timex.Gettext.pngettext/5","type":"macro"},{"doc":"","ref":"Timex.Gettext.html#pngettext_noop/3","title":"Timex.Gettext.pngettext_noop/3","type":"macro"},{"doc":"This module is used for creating and manipulating date/time intervals.Examplesiex&gt; use Timex ...&gt; Interval.new(from: ~D[2016-03-03], until: [days: 3]) %Elixir.Timex.Interval{from: ~N[2016-03-03 00:00:00], left_open: false, right_open: true, step: [days: 1], until: ~N[2016-03-06 00:00:00]} iex&gt; use Timex ...&gt; Interval.new(from: ~D[2016-03-03], until: ~N[2016-03-10 01:23:45]) %Timex.Interval{from: ~N[2016-03-03 00:00:00], left_open: false, right_open: true, step: [days: 1], until: ~N[2016-03-10 01:23:45]} iex&gt; use Timex ...&gt; ~N[2016-03-04 12:34:56] in Interval.new(from: ~D[2016-03-03], until: [days: 3]) true iex&gt; use Timex ...&gt; ~D[2016-03-01] in Interval.new(from: ~D[2016-03-03], until: [days: 3]) false iex&gt; use Timex ...&gt; Interval.overlaps?(Interval.new(from: ~D[2016-03-01], until: [days: 5]), Interval.new(from: ~D[2016-03-03], until: [days: 3])) true iex&gt; use Timex ...&gt; Interval.overlaps?(Interval.new(from: ~D[2016-03-01], until: [days: 1]), Interval.new(from: ~D[2016-03-03], until: [days: 3])) false","ref":"Timex.Interval.html","title":"Timex.Interval","type":"module"},{"doc":"Returns true if the first interval includes every point in time the second includes.Examplesiex&gt; Elixir.Timex.Interval.contains?(Elixir.Timex.Interval.new(from: ~D[2018-01-01], until: ~D[2018-01-31]), Elixir.Timex.Interval.new(from: ~D[2018-01-01], until: ~D[2018-01-30])) true iex&gt; Elixir.Timex.Interval.contains?(Elixir.Timex.Interval.new(from: ~D[2018-01-01], until: ~D[2018-01-30]), Elixir.Timex.Interval.new(from: ~D[2018-01-01], until: ~D[2018-01-31])) false iex&gt; Elixir.Timex.Interval.contains?(Elixir.Timex.Interval.new(from: ~D[2018-01-01], until: ~D[2018-01-10]), Elixir.Timex.Interval.new(from: ~D[2018-01-05], until: ~D[2018-01-15])) false","ref":"Timex.Interval.html#contains?/2","title":"Timex.Interval.contains?/2","type":"function"},{"doc":"Removes one interval from another which may reduce, split, or eliminate the original interval. Returns a (possibly empty) list of intervals representing the remaining time.GraphsThe following textual graphs show all the ways that the original interval and the removal can relate to each other, and the action that difference/2 will take in each case. The original interval is drawn with Os and the removal interval with Xs.# return original OO XX # trim end OOO XXX # trim end OOOOO XXX # split OOOOOOO XXX # eliminate OO XX # eliminate OO XXXX # trim beginning OOOO XX # eliminate OO XXXXXX # eliminate OO XXXXXX # trim beginning OOO XXX # return original OO XXExamplesiex&gt; Elixir.Timex.Interval.difference(Elixir.Timex.Interval.new(from: ~N[2018-01-01 02:00:00.000], until: ~N[2018-01-01 04:00:00.000]), Elixir.Timex.Interval.new(from: ~N[2018-01-01 03:00:00.000], until: ~N[2018-01-01 05:00:00.000])) [%Elixir.Timex.Interval{from: ~N[2018-01-01 02:00:00.000], left_open: false, right_open: true, step: [days: 1], until: ~N[2018-01-01 03:00:00.000]}] iex&gt; Elixir.Timex.Interval.difference(Elixir.Timex.Interval.new(from: ~N[2018-01-01 01:00:00.000], until: ~N[2018-01-01 05:00:00.000]), Elixir.Timex.Interval.new(from: ~N[2018-01-01 02:00:00.000], until: ~N[2018-01-01 03:00:00.000])) [%Elixir.Timex.Interval{from: ~N[2018-01-01 01:00:00.000], left_open: false, right_open: true, step: [days: 1], until: ~N[2018-01-01 02:00:00.000]}, %Elixir.Timex.Interval{from: ~N[2018-01-01 03:00:00.000], left_open: false, right_open: true, step: [days: 1], until: ~N[2018-01-01 05:00:00.000]}] iex&gt; Elixir.Timex.Interval.difference(Elixir.Timex.Interval.new(from: ~N[2018-01-01 02:00:00.000], until: ~N[2018-01-01 04:00:00.000]), Elixir.Timex.Interval.new(from: ~N[2018-01-01 01:00:00.000], until: ~N[2018-01-01 05:00:00.000])) []","ref":"Timex.Interval.html#difference/2","title":"Timex.Interval.difference/2","type":"function"},{"doc":"Return the interval duration, given a unit.When the unit is one of :seconds, :minutes, :hours, :days, :weeks, :months, :years, the result is an integer.When the unit is :duration, the result is a Duration struct.Exampleiex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [months: 5]) ...&gt; |&gt; Interval.duration(:months) 5 iex&gt; use Timex ...&gt; Interval.new(from: ~N[2014-09-22T15:30:00], until: [minutes: 20]) ...&gt; |&gt; Interval.duration(:duration) Duration.from_minutes(20)","ref":"Timex.Interval.html#duration/2","title":"Timex.Interval.duration/2","type":"function"},{"doc":"Formats the interval as a human readable string.Examplesiex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3]) ...&gt; |&gt; Interval.format!(&quot;%Y-%m-%d %H:%M&quot;, :strftime) &quot;[2014-09-22 00:00, 2014-09-25 00:00)&quot; iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3]) ...&gt; |&gt; Interval.format!(&quot;%Y-%m-%d&quot;, :strftime) &quot;[2014-09-22, 2014-09-25)&quot;","ref":"Timex.Interval.html#format/3","title":"Timex.Interval.format/3","type":"function"},{"doc":"Same as format/3, but raises a Timex.Interval.FormatError on failure.","ref":"Timex.Interval.html#format!/3","title":"Timex.Interval.format!/3","type":"function"},{"doc":"Create a new Interval struct.Note: By default intervals are left closed, i.e. they include the from date/time, and exclude the until date/time. Put another way, from &lt;= x &lt; until. This behavior matches that of other popular date/time libraries, such as Joda Time, as well as the SQL behavior of the overlaps keyword.Options:from: The date the interval starts at. Should be a (Naive)DateTime.until: Either a (Naive)DateTime, or a time shift that will be applied to the from date. This value must be greater than from, otherwise an error will be returned.left_open: Whether the interval is left open. See explanation below.right_open: Whether the interval is right open. See explanation below.step: The step to use when iterating the interval, defaults to [days: 1]The terms left_open and right_open come from the mathematical concept of intervals. You can see more detail on the theory on Wikipedia, but it can be more intuitively thought of like so:An &quot;open&quot; bound is exclusive, and a &quot;closed&quot; bound is inclusiveSo a left-closed interval includes the from value, and a left-open interval does not.Likewise, a right-closed interval includes the until value, and a right-open interval does not.An open interval is both left and right open, conversely, a closed interval is both left and right closed.Note: until shifts delegate to Timex.shift, so the options provided should match its valid options.Examplesiex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: ~D[2014-09-29]) ...&gt; |&gt; Interval.format!(&quot;%Y-%m-%d&quot;, :strftime) &quot;[2014-09-22, 2014-09-29)&quot; iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 7]) ...&gt; |&gt; Interval.format!(&quot;%Y-%m-%d&quot;, :strftime) &quot;[2014-09-22, 2014-09-29)&quot; iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 7], left_open: true, right_open: false) ...&gt; |&gt; Interval.format!(&quot;%Y-%m-%d&quot;, :strftime) &quot;(2014-09-22, 2014-09-29]&quot; iex&gt; use Timex ...&gt; Interval.new(from: ~N[2014-09-22T15:30:00], until: [minutes: 20], right_open: false) ...&gt; |&gt; Interval.format!(&quot;%H:%M&quot;, :strftime) &quot;[15:30, 15:50]&quot;","ref":"Timex.Interval.html#new/1","title":"Timex.Interval.new/1","type":"function"},{"doc":"Returns true if the first interval shares any point(s) in time with the second.Examplesiex&gt; Elixir.Timex.Interval.overlaps?(Elixir.Timex.Interval.new(from: ~D[2016-03-04], until: [days: 1]), Elixir.Timex.Interval.new(from: ~D[2016-03-03], until: [days: 3])) true iex&gt; Elixir.Timex.Interval.overlaps?(Elixir.Timex.Interval.new(from: ~D[2016-03-07], until: [days: 1]), Elixir.Timex.Interval.new(from: ~D[2016-03-03], until: [days: 3])) false","ref":"Timex.Interval.html#overlaps?/2","title":"Timex.Interval.overlaps?/2","type":"function"},{"doc":"Change the step value for the provided interval.The step should be a keyword list valid for use with Timex.Date.shift.Examplesiex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3], right_open: true) ...&gt; |&gt; Interval.with_step([days: 1]) |&gt; Enum.map(&amp;Timex.format!(&amp;1, &quot;%Y-%m-%d&quot;, :strftime)) [&quot;2014-09-22&quot;, &quot;2014-09-23&quot;, &quot;2014-09-24&quot;] iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3], right_open: false) ...&gt; |&gt; Interval.with_step([days: 1]) |&gt; Enum.map(&amp;Timex.format!(&amp;1, &quot;%Y-%m-%d&quot;, :strftime)) [&quot;2014-09-22&quot;, &quot;2014-09-23&quot;, &quot;2014-09-24&quot;, &quot;2014-09-25&quot;] iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3], right_open: false) ...&gt; |&gt; Interval.with_step([days: 2]) |&gt; Enum.map(&amp;Timex.format!(&amp;1, &quot;%Y-%m-%d&quot;, :strftime)) [&quot;2014-09-22&quot;, &quot;2014-09-24&quot;] iex&gt; use Timex ...&gt; Interval.new(from: ~D[2014-09-22], until: [days: 3], right_open: false) ...&gt; |&gt; Interval.with_step([days: 3]) |&gt; Enum.map(&amp;Timex.format!(&amp;1, &quot;%Y-%m-%d&quot;, :strftime)) [&quot;2014-09-22&quot;, &quot;2014-09-25&quot;]","ref":"Timex.Interval.html#with_step/2","title":"Timex.Interval.with_step/2","type":"function"},{"doc":"","ref":"Timex.Interval.html#t:t/0","title":"Timex.Interval.t/0","type":"type"},{"doc":"","ref":"Timex.Interval.html#t:valid_interval_step/0","title":"Timex.Interval.valid_interval_step/0","type":"type"},{"doc":"","ref":"Timex.Interval.html#t:valid_interval_steps/0","title":"Timex.Interval.valid_interval_steps/0","type":"type"},{"doc":"","ref":"Timex.Interval.html#t:valid_step_unit/0","title":"Timex.Interval.valid_step_unit/0","type":"type"},{"doc":"","ref":"Timex.NaiveDateTime.html","title":"Timex.NaiveDateTime","type":"module"},{"doc":"This is the base plugin behavior for all Timex date/time string parsers.","ref":"Timex.Parse.DateTime.Parser.html","title":"Timex.Parse.DateTime.Parser","type":"module"},{"doc":"Parses a date/time string using the default parser.Examplesiex&gt; use Timex ...&gt; {:ok, dt} = Elixir.Timex.Parse.DateTime.Parser.parse(&quot;2014-07-29T00:20:41.196Z&quot;, &quot;{ISO:Extended:Z}&quot;) ...&gt; dt.year 2014 iex&gt; dt.month 7 iex&gt; dt.day 29 iex&gt; dt.time_zone &quot;Etc/UTC&quot;","ref":"Timex.Parse.DateTime.Parser.html#parse/2","title":"Timex.Parse.DateTime.Parser.parse/2","type":"function"},{"doc":"Parses a date/time string using the provided tokenizer. Tokenizers must implement the Timex.Parse.DateTime.Tokenizer behaviour.Examplesiex&gt; use Timex ...&gt; {:ok, dt} = Elixir.Timex.Parse.DateTime.Parser.parse(&quot;2014-07-29T00:30:41.196-02:00&quot;, &quot;{ISO:Extended}&quot;, Timex.Parse.DateTime.Tokenizers.Default) ...&gt; dt.year 2014 iex&gt; dt.month 7 iex&gt; dt.day 29 iex&gt; dt.time_zone &quot;Etc/UTC-2&quot;","ref":"Timex.Parse.DateTime.Parser.html#parse/3","title":"Timex.Parse.DateTime.Parser.parse/3","type":"function"},{"doc":"Same as parse/2 and parse/3, but raises on error.","ref":"Timex.Parse.DateTime.Parser.html#parse!/3","title":"Timex.Parse.DateTime.Parser.parse!/3","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended","type":"module"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse/1","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse/1","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse_extended/1","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse_extended/1","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse_extended/4","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse_extended/4","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse_fractional_seconds/3","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse_fractional_seconds/3","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse_offset/3","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse_offset/3","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#parse_offset/4","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parse_offset/4","type":"function"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#t:parser/0","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.parser/0","type":"type"},{"doc":"","ref":"Timex.Parse.DateTime.Parsers.ISO8601Extended.html#t:previous_parser/0","title":"Timex.Parse.DateTime.Parsers.ISO8601Extended.previous_parser/0","type":"type"},{"doc":"Defines the API for a custom tokenizer which can extend Timex's datetime parsing facilities.","ref":"Timex.Parse.DateTime.Tokenizer.html","title":"Timex.Parse.DateTime.Tokenizer","type":"behaviour"},{"doc":"","ref":"Timex.Parse.DateTime.Tokenizer.html#c:apply/3","title":"Timex.Parse.DateTime.Tokenizer.apply/3","type":"callback"},{"doc":"","ref":"Timex.Parse.DateTime.Tokenizer.html#c:tokenize/1","title":"Timex.Parse.DateTime.Tokenizer.tokenize/1","type":"callback"},{"doc":"Implements the parser for the default DateTime format strings.","ref":"Timex.Parse.DateTime.Tokenizers.Default.html","title":"Timex.Parse.DateTime.Tokenizers.Default","type":"module"},{"doc":"Applies a given token + value to the DateTime represented by the current input string.","ref":"Timex.Parse.DateTime.Tokenizers.Default.html#apply/3","title":"Timex.Parse.DateTime.Tokenizers.Default.apply/3","type":"function"},{"doc":"Tokenizes the given format string and returns an error or a list of directives.","ref":"Timex.Parse.DateTime.Tokenizers.Default.html#tokenize/1","title":"Timex.Parse.DateTime.Tokenizers.Default.tokenize/1","type":"function"},{"doc":"Implements the parser for strftime-style datetime format strings.","ref":"Timex.Parse.DateTime.Tokenizers.Strftime.html","title":"Timex.Parse.DateTime.Tokenizers.Strftime","type":"module"},{"doc":"Applies a given token + value to the DateTime represented by the current input string.","ref":"Timex.Parse.DateTime.Tokenizers.Strftime.html#apply/3","title":"Timex.Parse.DateTime.Tokenizers.Strftime.apply/3","type":"function"},{"doc":"Tokenizes the given format string and returns an error or a list of directives.","ref":"Timex.Parse.DateTime.Tokenizers.Strftime.html#tokenize/1","title":"Timex.Parse.DateTime.Tokenizers.Strftime.tokenize/1","type":"function"},{"doc":"This module is responsible for parsing input strings into Duration structs. The actual parsing is delegated to specific parser modules, but this module provides a unified API for all of them.","ref":"Timex.Parse.Duration.Parser.html","title":"Timex.Parse.Duration.Parser","type":"behaviour"},{"doc":"Parses the given input using the ISO-8601 duration parser, and returns either an :ok, or :error tuple.","ref":"Timex.Parse.Duration.Parser.html#parse/1","title":"Timex.Parse.Duration.Parser.parse/1","type":"function"},{"doc":"","ref":"Timex.Parse.Duration.Parser.html#c:parse/1","title":"Timex.Parse.Duration.Parser.parse/1","type":"callback"},{"doc":"Parses the given input using the provided parser module, and returns either an :ok, or :error tuple.","ref":"Timex.Parse.Duration.Parser.html#parse/2","title":"Timex.Parse.Duration.Parser.parse/2","type":"function"},{"doc":"Parses the given input using the ISO-8601 duration parser, and either returns a Duration, or raises an error.","ref":"Timex.Parse.Duration.Parser.html#parse!/1","title":"Timex.Parse.Duration.Parser.parse!/1","type":"function"},{"doc":"Parses the given input using the provided parser module, and either returns a Duration, or raises an error.","ref":"Timex.Parse.Duration.Parser.html#parse!/2","title":"Timex.Parse.Duration.Parser.parse!/2","type":"function"},{"doc":"This module parses ISO-8601 duration strings into Duration structs.","ref":"Timex.Parse.Duration.Parsers.ISO8601Parser.html","title":"Timex.Parse.Duration.Parsers.ISO8601Parser","type":"module"},{"doc":"Parses an ISO-8601 formatted duration string into a Duration struct. The parse result is wrapped in a :ok/:error tuple.Examplesiex&gt; {:ok, d} = Elixir.Timex.Parse.Duration.Parsers.ISO8601Parser.parse(&quot;P15Y3M2DT1H14M37.25S&quot;) ...&gt; Timex.Format.Duration.Formatter.format(d) &quot;P15Y3M2DT1H14M37.25S&quot; iex&gt; {:ok, d} = Elixir.Timex.Parse.Duration.Parsers.ISO8601Parser.parse(&quot;P15Y3M2D&quot;) ...&gt; Timex.Format.Duration.Formatter.format(d) &quot;P15Y3M2D&quot; iex&gt; {:ok, d} = Elixir.Timex.Parse.Duration.Parsers.ISO8601Parser.parse(&quot;PT3H12M25.001S&quot;) ...&gt; Timex.Format.Duration.Formatter.format(d) &quot;PT3H12M25.001S&quot; iex&gt; {:ok, d} = Elixir.Timex.Parse.Duration.Parsers.ISO8601Parser.parse(&quot;P2W&quot;) ...&gt; Timex.Format.Duration.Formatter.format(d) &quot;P14D&quot; iex&gt; Elixir.Timex.Parse.Duration.Parsers.ISO8601Parser.parse(&quot;P15YT3D&quot;) {:error, &quot;invalid use of date component after time separator&quot;}","ref":"Timex.Parse.Duration.Parsers.ISO8601Parser.html#parse/1","title":"Timex.Parse.Duration.Parsers.ISO8601Parser.parse/1","type":"function"},{"doc":"Parses POSIX-style timezones:FormatPOSIX-style timezones are of the format: local_timezone,date/time,date/timeWhere date is in the Mm.n.d format, and where:Mm (1-12) for 12 monthsn (1-5) 1 for the first week and 5 for the last week in the monthd (0-6) 0 for Sunday and 6 for SaturdayExampleTZ = CST6CDT,M3.2.0/2:00:00,M11.1.0/2:00:00This would represents a change to daylight saving time at 2:00 AM on the second Sunday in March and change back at 2:00 AM on the first Sunday in November, and keep 6 hours time offset from GMT every year. The breakdown of the string is:CST6CDT is the timezone nameCST is the standard abbreviation6 is the hours of time difference from GMTCDT is the DST abbreviation,M3 is the third month.2 is the second occurrence of the day in the month.0 is Sunday/2:00:00 is the time,M11 is the eleventh month.1 is the first occurrence of the day in the month.0 is Sunday/2:00:00 is the time","ref":"Timex.Parse.Timezones.Posix.html","title":"Timex.Parse.Timezones.Posix","type":"module"},{"doc":"","ref":"Timex.Parse.Timezones.Posix.html#parse/1","title":"Timex.Parse.Timezones.Posix.parse/1","type":"function"},{"doc":"","ref":"Timex.Parse.Timezones.Posix.PosixTimezone.html","title":"Timex.Parse.Timezones.Posix.PosixTimezone","type":"module"},{"doc":"Specdst_start/dst_endmonth: 1-12week: week of the monthday_of_week: 0-6, 0 is Sundaytime: {hour, minute, second}","ref":"Timex.Parse.Timezones.Posix.PosixTimezone.html#__struct__/0","title":"Timex.Parse.Timezones.Posix.PosixTimezone.__struct__/0","type":"function"},{"doc":"This module is responsible for parsing binary zoneinfo files, such as those found in /usr/local/zoneinfo.","ref":"Timex.Parse.ZoneInfo.Parser.html","title":"Timex.Parse.ZoneInfo.Parser","type":"module"},{"doc":"Parses a binary representing a valid zoneinfo file.Parses the timezone information inside, and returns it as a Zone struct.","ref":"Timex.Parse.ZoneInfo.Parser.html#parse/1","title":"Timex.Parse.ZoneInfo.Parser.parse/1","type":"function"},{"doc":"Like parse/1, but expects a file path to parse.","ref":"Timex.Parse.ZoneInfo.Parser.html#parse_file/1","title":"Timex.Parse.ZoneInfo.Parser.parse_file/1","type":"function"},{"doc":"Represents the data retreived from a binary tzfile.","ref":"Timex.Parse.ZoneInfo.Parser.Zone.html","title":"Timex.Parse.ZoneInfo.Parser.Zone","type":"module"},{"doc":"This protocol defines the API for functions which take a Date, NaiveDateTime, or DateTime as input.","ref":"Timex.Protocol.html","title":"Timex.Protocol","type":"protocol"},{"doc":"Get a new version of the date/time value representing the beginning of the day","ref":"Timex.Protocol.html#beginning_of_day/1","title":"Timex.Protocol.beginning_of_day/1","type":"function"},{"doc":"Get a new version of the date/time value representing the beginning of it's month","ref":"Timex.Protocol.html#beginning_of_month/1","title":"Timex.Protocol.beginning_of_month/1","type":"function"},{"doc":"Get a new version of the date/time value representing the beginning of it's quarter","ref":"Timex.Protocol.html#beginning_of_quarter/1","title":"Timex.Protocol.beginning_of_quarter/1","type":"function"},{"doc":"Get a new version of the date/time value representing the beginning of it's week, providing a weekday name (as an atom) for the day which starts the week, i.e. :mon.","ref":"Timex.Protocol.html#beginning_of_week/2","title":"Timex.Protocol.beginning_of_week/2","type":"function"},{"doc":"Get a new version of the date/time value representing the beginning of it's year","ref":"Timex.Protocol.html#beginning_of_year/1","title":"Timex.Protocol.beginning_of_year/1","type":"function"},{"doc":"Get the century a date/time value is in","ref":"Timex.Protocol.html#century/1","title":"Timex.Protocol.century/1","type":"function"},{"doc":"Get the ordinal day number of the given date/time value","ref":"Timex.Protocol.html#day/1","title":"Timex.Protocol.day/1","type":"function"},{"doc":"Get the number of days in the month for the given date/time value","ref":"Timex.Protocol.html#days_in_month/1","title":"Timex.Protocol.days_in_month/1","type":"function"},{"doc":"Get a new version of the date/time value representing the end of the day","ref":"Timex.Protocol.html#end_of_day/1","title":"Timex.Protocol.end_of_day/1","type":"function"},{"doc":"Get a new version of the date/time value representing the ending of it's month","ref":"Timex.Protocol.html#end_of_month/1","title":"Timex.Protocol.end_of_month/1","type":"function"},{"doc":"Get a new version of the date/time value representing the ending of it's quarter","ref":"Timex.Protocol.html#end_of_quarter/1","title":"Timex.Protocol.end_of_quarter/1","type":"function"},{"doc":"Get a new version of the date/time value representing the ending of it's week, providing a weekday name (as an atom) for the day which starts the week, i.e. :mon.","ref":"Timex.Protocol.html#end_of_week/2","title":"Timex.Protocol.end_of_week/2","type":"function"},{"doc":"Get a new version of the date/time value representing the ending of it's year","ref":"Timex.Protocol.html#end_of_year/1","title":"Timex.Protocol.end_of_year/1","type":"function"},{"doc":"Shifts the given date/time value to the ISO day given","ref":"Timex.Protocol.html#from_iso_day/2","title":"Timex.Protocol.from_iso_day/2","type":"function"},{"doc":"Return a boolean indicating whether the date/time value is in a leap year","ref":"Timex.Protocol.html#is_leap?/1","title":"Timex.Protocol.is_leap?/1","type":"function"},{"doc":"Determine if the provided date/time value is valid.","ref":"Timex.Protocol.html#is_valid?/1","title":"Timex.Protocol.is_valid?/1","type":"function"},{"doc":"Return a pair {year, week number} (as defined by ISO 8601) that the given date/time value falls on.","ref":"Timex.Protocol.html#iso_week/1","title":"Timex.Protocol.iso_week/1","type":"function"},{"doc":"Get the quarter for the given date/time value","ref":"Timex.Protocol.html#quarter/1","title":"Timex.Protocol.quarter/1","type":"function"},{"doc":"Set fields on a date/time value using a list of unit/value pairs","ref":"Timex.Protocol.html#set/2","title":"Timex.Protocol.set/2","type":"function"},{"doc":"Shift a date/time value using a list of shift unit/value pairs","ref":"Timex.Protocol.html#shift/2","title":"Timex.Protocol.shift/2","type":"function"},{"doc":"Convert a date/time value to a Date","ref":"Timex.Protocol.html#to_date/1","title":"Timex.Protocol.to_date/1","type":"function"},{"doc":"Convert a date/time value to a DateTime. An optional timezone can be provided, UTC will be assumed if one is not provided.","ref":"Timex.Protocol.html#to_datetime/2","title":"Timex.Protocol.to_datetime/2","type":"function"},{"doc":"Convert a date/time value to it's Erlang tuple variant i.e. Date becomes {y,m,d}, DateTime/NaiveDateTime become {{y,m,d},{h,mm,s}}","ref":"Timex.Protocol.html#to_erl/1","title":"Timex.Protocol.to_erl/1","type":"function"},{"doc":"Convert a date/time value to gregorian microseconds (microseconds since the start of year zero)","ref":"Timex.Protocol.html#to_gregorian_microseconds/1","title":"Timex.Protocol.to_gregorian_microseconds/1","type":"function"},{"doc":"Convert a date/time value to gregorian seconds (seconds since start of year zero)","ref":"Timex.Protocol.html#to_gregorian_seconds/1","title":"Timex.Protocol.to_gregorian_seconds/1","type":"function"},{"doc":"Convert a date/time value to a Julian calendar date number","ref":"Timex.Protocol.html#to_julian/1","title":"Timex.Protocol.to_julian/1","type":"function"},{"doc":"Convert a date/time value to a NaiveDateTime","ref":"Timex.Protocol.html#to_naive_datetime/1","title":"Timex.Protocol.to_naive_datetime/1","type":"function"},{"doc":"Convert a date/time value to seconds since the UNIX Epoch","ref":"Timex.Protocol.html#to_unix/1","title":"Timex.Protocol.to_unix/1","type":"function"},{"doc":"Get the week number of the given date/time value, starting at 1","ref":"Timex.Protocol.html#week_of_month/1","title":"Timex.Protocol.week_of_month/1","type":"function"},{"doc":"Get the ordinal weekday number of the given date/time value","ref":"Timex.Protocol.html#weekday/1","title":"Timex.Protocol.weekday/1","type":"function"},{"doc":"Get the ordinal weekday number of the given date/time value and relative to the given weekstart","ref":"Timex.Protocol.html#weekday/2","title":"Timex.Protocol.weekday/2","type":"function"},{"doc":"","ref":"Timex.Protocol.html#t:t/0","title":"Timex.Protocol.t/0","type":"type"},{"doc":"This module provides helper functions for working with Times","ref":"Timex.Time.html","title":"Timex.Time","type":"module"},{"doc":"Converts an hour between 0..24 to {1..12, :am/:pm}Examplesiex&gt; Timex.Time.to_12hour_clock(23) {11, :pm}","ref":"Timex.Time.html#to_12hour_clock/1","title":"Timex.Time.to_12hour_clock/1","type":"function"},{"doc":"Converts an hour between 1..12 in either am or pm, to value between 0..24Examplesiex&gt; Timex.Time.to_24hour_clock(7, :pm) 19","ref":"Timex.Time.html#to_24hour_clock/2","title":"Timex.Time.to_24hour_clock/2","type":"function"},{"doc":"This module is used for looking up the timezone information for a given point in time, in the desired zone. Timezones are dependent not only on locale, but the date and time for which you are querying. For instance, the timezone offset from UTC for Europe/Moscow is different for March 3rd of 2015, than it was in 2013. These differences are important, and as such, all functions in this module are date/time sensitive, and where omitted, the current date/time are assumed.In addition to lookups, this module also does conversion of datetimes from one timezone period to another, and determining the difference between a date in one timezone period and the same date/time in another timezone period.","ref":"Timex.Timezone.html","title":"Timex.Timezone","type":"module"},{"doc":"Shifts the provided DateTime to the beginning of the day in it's timezone","ref":"Timex.Timezone.html#beginning_of_day/1","title":"Timex.Timezone.beginning_of_day/1","type":"function"},{"doc":"Convert a date to the given timezone (either TimezoneInfo or a timezone name)","ref":"Timex.Timezone.html#convert/2","title":"Timex.Timezone.convert/2","type":"function"},{"doc":"Shifts the provided DateTime to the end of the day in it's timezone","ref":"Timex.Timezone.html#end_of_day/1","title":"Timex.Timezone.end_of_day/1","type":"function"},{"doc":"Determines if a given zone name exists","ref":"Timex.Timezone.html#exists?/1","title":"Timex.Timezone.exists?/1","type":"function"},{"doc":"Gets timezone info for a given zone name and date. The date provided can either be an Erlang datetime tuple, or a DateTime struct, and if one is not provided, then the current date and time is returned.","ref":"Timex.Timezone.html#get/2","title":"Timex.Timezone.get/2","type":"function"},{"doc":"Gets the local timezone configuration for the current date and time.","ref":"Timex.Timezone.html#local/0","title":"Timex.Timezone.local/0","type":"function"},{"doc":"Gets the local timezone configuration for the provided date and time. The provided date and time can either be an Erlang datetime tuple, or a DateTime struct.","ref":"Timex.Timezone.html#local/1","title":"Timex.Timezone.local/1","type":"function"},{"doc":"This function takes one of the varying timezone representations:atomsoffset integersshortcut names (i.e. :utc, :local, &quot;Z&quot;, &quot;A&quot;)and resolves the full name of the timezone if it's able.If a string is provided which isn't recognized, it is returned untouched, only when get/2 is called will the timezone lookup fail.","ref":"Timex.Timezone.html#name_of/1","title":"Timex.Timezone.name_of/1","type":"function"},{"doc":"Given a timezone name as a string, and a date/time in the form of the number of seconds since year zero, attempt to resolve a TimezoneInfo for that date/time. If the time is ambiguous, AmbiguousTimezoneInfo will be returned. If no result is found, an error will be returned.If an invalid zone name is provided, an error will be returned","ref":"Timex.Timezone.html#resolve/3","title":"Timex.Timezone.resolve/3","type":"function"},{"doc":"","ref":"Timex.Timezone.html#total_offset/1","title":"Timex.Timezone.total_offset/1","type":"function"},{"doc":"","ref":"Timex.Timezone.html#total_offset/2","title":"Timex.Timezone.total_offset/2","type":"function"},{"doc":"","ref":"Timex.Timezone.html#tzdata_to_timezone/2","title":"Timex.Timezone.tzdata_to_timezone/2","type":"function"},{"doc":"This module is responsible for determining the timezone configuration of the local machine. It determines this from a number of sources, depending on platform, but the order of precedence is as follows:ALL:TZ environment variable. Ignored if nil/emptyOSX:/etc/localtimesystemsetup -gettimezone (if admin rights are present)UNIX:/etc/timezone/etc/sysconfig/clock/etc/conf.d/clock/etc/localtime/usr/local/etc/localtimeWindows:SYSTEM registry for the currently configured TimeZoneInformationEach location is tried, and if an error is encountered, the next is attempted, until either a successful lookup is performed, or we run out of locations to check.","ref":"Timex.Timezone.Local.html","title":"Timex.Timezone.Local","type":"module"},{"doc":"Looks up the local timezone configuration. Returns the name of a timezone in the Olson database.If no reference time is provided (in gregorian seconds), the current time in UTC will be used. If one is provided, the reference time will be used to find the local timezone for that reference time, if it exists.","ref":"Timex.Timezone.Local.html#lookup/0","title":"Timex.Timezone.Local.lookup/0","type":"function"},{"doc":"Given a binary representing the data from a tzfile (not the source version), parses out the timezone for the curent date/time in UTC.","ref":"Timex.Timezone.Local.html#parse_tzfile/1","title":"Timex.Timezone.Local.parse_tzfile/1","type":"function"},{"doc":"","ref":"Timex.Timezone.Local.html#t:gregorian_seconds/0","title":"Timex.Timezone.Local.gregorian_seconds/0","type":"type"},{"doc":"All relevant timezone information for a given period, i.e. Europe/Moscow on March 3rd, 2013Notes:full_name is the name of the zone, but does not indicate anything about the current period (i.e. CST vs CDT)abbreviation is the abbreviated name for the zone in the current period, i.e. &quot;America/Chicago&quot; on 3/30/15 is &quot;CDT&quot;offset_std is the offset in seconds from standard time for this periodoffset_utc is the offset in seconds from UTC for this periodSpec:day_of_week: :sunday, :monday, :tuesday, etcdatetime: {{year, month, day}, {hour, minute, second}}from: :min | {day_of_week, datetime}, when this zone startsuntil: :max | {day_of_week, datetime}, when this zone ends","ref":"Timex.TimezoneInfo.html","title":"Timex.TimezoneInfo","type":"module"},{"doc":"Create a custom timezone if a built-in one does not meet your needs.You must provide the name, abbreviation, offset from UTC, daylight savings time offset, and the from/until reference points for when the zone takes effect and ends.To clarify the two offsets, offset_utc is the absolute offset relative to UTC, offset_std is the offset to apply to offset_utc which gives us the offset from UTC during daylight savings time for this timezone. If DST does not apply for this zone, simply set it to 0.The from/until reference points must meet the following criteria:- Be set to `:min` for from, or `:max` for until, which represent &quot;infinity&quot; for the start/end of the zone period. - OR, be a tuple of {day_of_week, datetime}, where: - `day_of_week` is an atom like `:sunday` - `datetime` is an Erlang datetime tuple, e.g. `{{2016,10,8},{2,0,0}}`IMPORTANT: Offsets are in seconds, not minutes, if you do not ensure they are in the correct unit, runtime errors or incorrect results are probable.Examplesiex&gt; Elixir.Timex.TimezoneInfo.create(&quot;Etc/Test&quot;, &quot;TST&quot;, 120*60, 0, :min, :max) %TimezoneInfo{full_name: &quot;Etc/Test&quot;, abbreviation: &quot;TST&quot;, offset_std: 7200, offset_utc: 0, from: :min, until: :max} ...&gt; Elixir.Timex.TimezoneInfo.create(&quot;Etc/Test&quot;, &quot;TST&quot;, 24*60*60, 0, :min, :max) {:error, &quot;invalid timezone offset &#39;86400&#39;&quot;}","ref":"Timex.TimezoneInfo.html#create/6","title":"Timex.TimezoneInfo.create/6","type":"function"},{"doc":"","ref":"Timex.TimezoneInfo.html#from_datetime/1","title":"Timex.TimezoneInfo.from_datetime/1","type":"function"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:datetime/0","title":"Timex.TimezoneInfo.datetime/0","type":"type"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:day_of_week/0","title":"Timex.TimezoneInfo.day_of_week/0","type":"type"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:from_constraint/0","title":"Timex.TimezoneInfo.from_constraint/0","type":"type"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:offset/0","title":"Timex.TimezoneInfo.offset/0","type":"type"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:t/0","title":"Timex.TimezoneInfo.t/0","type":"type"},{"doc":"","ref":"Timex.TimezoneInfo.html#t:until_constraint/0","title":"Timex.TimezoneInfo.until_constraint/0","type":"type"},{"doc":"","ref":"Timex.Translator.html","title":"Timex.Translator","type":"module"},{"doc":"Returns the active locale for the process in which this function is called","ref":"Timex.Translator.html#current_locale/0","title":"Timex.Translator.current_locale/0","type":"function"},{"doc":"Returns a map of day period types to translated day period namesExamplesiex&gt; day_periods = Timex.Translator.get_day_periods(&quot;en&quot;) ...&gt; {day_periods[:am], day_periods[:AM]} {&quot;am&quot;, &quot;AM&quot;}","ref":"Timex.Translator.html#get_day_periods/1","title":"Timex.Translator.get_day_periods/1","type":"function"},{"doc":"Returns a map of ordinal months to month names","ref":"Timex.Translator.html#get_months/1","title":"Timex.Translator.get_months/1","type":"function"},{"doc":"Returns a map of ordinal months to month abbreviations","ref":"Timex.Translator.html#get_months_abbreviated/1","title":"Timex.Translator.get_months_abbreviated/1","type":"function"},{"doc":"Returns a map of ordinal weekdays to weekday names, where Monday = 1, translated in the given locale","ref":"Timex.Translator.html#get_weekdays/1","title":"Timex.Translator.get_weekdays/1","type":"function"},{"doc":"Returns a map of ordinal weekdays to weekday abbreviations, where Mon = 1","ref":"Timex.Translator.html#get_weekdays_abbreviated/1","title":"Timex.Translator.get_weekdays_abbreviated/1","type":"function"},{"doc":"Translates a string for a given locale and domain.Examplesiex&gt; Timex.Translator.translate(&quot;ru&quot;, &quot;weekdays&quot;, &quot;Saturday&quot;) &quot;суббота&quot; iex&gt; Timex.Translator.translate(&quot;it&quot;, &quot;weekdays&quot;, &quot;Saturday&quot;) &quot;Sabato&quot; iex&gt; Timex.Translator.translate(&quot;invalid_locale&quot;, &quot;weekdays&quot;, &quot;Saturday&quot;) &quot;Saturday&quot;","ref":"Timex.Translator.html#translate/3","title":"Timex.Translator.translate/3","type":"function"},{"doc":"Translates a string for a given locale and domain, following the pluralization rules of that language.Examplesiex&gt; Timex.Translator.translate_plural(&quot;ru&quot;, &quot;relative_time&quot;, &quot;in %{count} second&quot;, &quot;in %{count} seconds&quot;, 5) &quot;через 5 секунд&quot; iex&gt; Timex.Translator.translate_plural(&quot;it&quot;, &quot;relative_time&quot;, &quot;in %{count} second&quot;, &quot;in %{count} seconds&quot;, 5) &quot;in 5 secondi&quot; iex&gt; Timex.Translator.translate_plural(&quot;invalid_locale&quot;, &quot;relative_time&quot;, &quot;in %{count} second&quot;, &quot;in %{count} seconds&quot;, 5) &quot;in 5 seconds&quot;","ref":"Timex.Translator.html#translate_plural/5","title":"Timex.Translator.translate_plural/5","type":"function"},{"doc":"","ref":"Timex.Translator.html#with_locale/2","title":"Timex.Translator.with_locale/2","type":"macro"},{"doc":"","ref":"Timex.Types.html","title":"Timex.Types","type":"module"},{"doc":"","ref":"Timex.Types.html#t:calendar_types/0","title":"Timex.Types.calendar_types/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:date/0","title":"Timex.Types.date/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:datetime/0","title":"Timex.Types.datetime/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:day/0","title":"Timex.Types.day/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:daynum/0","title":"Timex.Types.daynum/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:hour/0","title":"Timex.Types.hour/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:iso_triplet/0","title":"Timex.Types.iso_triplet/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:megaseconds/0","title":"Timex.Types.megaseconds/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:microsecond/0","title":"Timex.Types.microsecond/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:microsecond_datetime/0","title":"Timex.Types.microsecond_datetime/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:microsecond_time/0","title":"Timex.Types.microsecond_time/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:microseconds/0","title":"Timex.Types.microseconds/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:minute/0","title":"Timex.Types.minute/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:month/0","title":"Timex.Types.month/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:num_of_days/0","title":"Timex.Types.num_of_days/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:second/0","title":"Timex.Types.second/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:seconds/0","title":"Timex.Types.seconds/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:shift_units/0","title":"Timex.Types.shift_units/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:std_offset/0","title":"Timex.Types.std_offset/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:time/0","title":"Timex.Types.time/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:time_units/0","title":"Timex.Types.time_units/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:time_zone/0","title":"Timex.Types.time_zone/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:timestamp/0","title":"Timex.Types.timestamp/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:tz_offset/0","title":"Timex.Types.tz_offset/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:utc_offset/0","title":"Timex.Types.utc_offset/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:valid_date/0","title":"Timex.Types.valid_date/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:valid_datetime/0","title":"Timex.Types.valid_datetime/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:valid_time/0","title":"Timex.Types.valid_time/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:valid_timezone/0","title":"Timex.Types.valid_timezone/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:week_of_month/0","title":"Timex.Types.week_of_month/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:weekday/0","title":"Timex.Types.weekday/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:weekday_name/0","title":"Timex.Types.weekday_name/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:weeknum/0","title":"Timex.Types.weeknum/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:weekstart/0","title":"Timex.Types.weekstart/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:year/0","title":"Timex.Types.year/0","type":"type"},{"doc":"","ref":"Timex.Types.html#t:zone_abbr/0","title":"Timex.Types.zone_abbr/0","type":"type"},{"doc":"Used for errors encountered during string formatting.","ref":"Timex.Format.FormatError.html","title":"Timex.Format.FormatError","type":"exception"},{"doc":"Thrown when an error occurs with formatting an Interval","ref":"Timex.Interval.FormatError.html","title":"Timex.Interval.FormatError","type":"exception"},{"doc":"Used for errors encountered during parsing.","ref":"Timex.Parse.ParseError.html","title":"Timex.Parse.ParseError","type":"exception"},{"doc":"Getting StartedLearn how to add timex to your Elixir project and start using it.If you are looking for a general reference of what functions are available to you, take a look at the Timex, Timex.Interval, Timex.Duration modules, as they cover the vast majority of functionality you will care about.","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"To use Timex with your projects, edit your mix.exs file and add it as a dependency, as well as add :timex to your applications list.defp deps do [{:timex, &quot;~&gt; 3.0&quot;}] endTo use Timex modules without the Timex namespace, add use Timex to the top of each module you plan on referencing Timex from. You can then reference the modules directly, for example: Duration.now, versus Timex.Duration.now. You can also alias individual modules if that suits your needs better, but for purposes of this documentation, we'll assume that you're going the use Timex route.What Is TimexTimex aims to be the richest, most comprehensive date/time library for Elixir projects, with the ultimate goal of being merged into the standard library, if such functionality is ever considered for inclusion.A rough list of current features:create datetimes in an arbitrary timezonecreate datetimes from Erlang dates or datetimesget dates/datetimes representing special points in time:end/beginning of yearend/beginning of quarterend/beginning of monthend/beginning of weekend/beginning of dayconvert to/from dates/datetimes in various units (seconds, minutes, hours, etc.) since year zero or UNIX epochconvert to/from dates/datetimes in various standard ISO forms:ISO tripletsISO dayISO weekconvert to/from dates/datetimes in various formats:gregoriangregorian_secondsErlang dates or datetimesUNIX timestampsshift datetimes across timezonesshift dates/datetimes through time using various unitsformat/parse datetime stringscompare and diff dates/datetimesnormalize dates/datetimescreate intervals between points in time which can be enumeratedcreate timestamps and convert them to other time unitscompare and diff timestamps and various time unitsmeasure execution time of a functionadd and subtract time or datetimesand more..","ref":"getting-started.html#project-setup","title":"Getting Started - Project Setup","type":"extras"},{"doc":"Change LogAll notable changes to this project will be documented in this file (at least to the extent possible, I am not infallible sadly). This project adheres to Semantic Versioning.","ref":"changelog.html","title":"Change Log","type":"extras"},{"doc":"NOTE: The config of the Timex default locale is changed to:config :timex, Timex.Gettext, default_locale: &quot;en&quot;This follows the standard set by Gettext, see: the Gettext docsNow when the Gettext locale is being changed on runtime with:Gettext.put_locale(&quot;nl&quot;)The Gettext backend for Timex will follow suit. If for some reason you want Timex, and just Timex, to change its locale to something else you should target the Timex.Gettext backend with:Timex.Gettext.put_locale(&quot;de&quot;)Potentially BreakingFixed handling of Etc/GMT vs GMT timezones. The former must be POSIX compatible, which inverts the meaning of the sign, whereas the latter have no such restriction and are equivalent to UTC(+/-)HH:MM, while this is fixing incorrect behavior, it could potentially break users relying on the incorrect behavior.Fixed#494, incorrect handling of parsing week numbersImprove parsing of timezonesIncorrect stringification of error values returned in some circumstancesLet Timex follow the locale set by the global Gettext. See #501Added/ChangedRemoved Timex.Timezone.diff, as it is no longer used in TimexFixed","ref":"changelog.html#3-7-2","title":"Change Log - 3.7.2","type":"extras"},{"doc":"Fixedlocal/0 and local/1 were incorrectly returning TimezoneInfo structs due to a bad match","ref":"changelog.html#3-7-1","title":"Change Log - 3.7.1","type":"extras"},{"doc":"Potentially BreakingElixir 1.8+ is now requiredTzdata 1.0+ is now requiredIf you were previously relying on ? suffixed functions to return {:error, reason} if given invalid date/time inputs (other than is_valid?), these functions now always return booleans and raise if an error with the input is encounteredAdded/ChangedRefactored much of the library to delegate to the Calendar API where appropriate, we now make more of an effort to avoid duplication of the standard library functionalityFunctions with the ? suffix now correctly raise on invalid inputs, and always return booleans. This was implicitly broken before, we need to follow convention here.Local timezone handling no longer requires parsing the zoneinfo files, instead we attempt to observe the timezone name that is active and feed that into the timezone database directly. We were just using the abbreviations before, but that wasn't correct behavior at all. In the future we may want to support the system timezone database as a proper implementation of Calendar.TimeZoneDatabase, but for now we've just removed the unnecessary parsing work that was going on here.Timex.today/1 which returns today's date in the provided timezoneFixedHandling of timezones across DST. More generally we now handle gaps/ambiguity much more consistentlyZoneInfo parser was refactored, now properly supports version 2/3 files, addresses some small bugs in previous codeSome incorrect/redundant typespecs were removed/fixedWe now support alternative timezone databases for API operations that do not need to interact with the Timex.Timezone module directly. That module is still tied to tzdata for now, but in the future may be modified to remove the direct dependency.","ref":"changelog.html#3-7-0","title":"Change Log - 3.7.0","type":"extras"},{"doc":"Potentially BreakingTimex.set/2 now sets microseconds when setting :time from a %Time{} structTimex.Duration.to_string/1 now returns PT0S instead of P for zero-length durations with the default formatterAddedTimex.set/2 now also accepts setting the :date from a %Date{} and :time from a %Time{} struct for NaiveDateTimeFixesLoosen tzdata dependency to allow 1.x releases","ref":"changelog.html#3-6-4","title":"Change Log - 3.6.4","type":"extras"},{"doc":"AddedSwitched to GitHub Actions for CIFinnish translationsEnumerable implementation for Timex.IntervalFixedCzech translation fixes#616#615","ref":"changelog.html#3-6-3","title":"Change Log - 3.6.3","type":"extras"},{"doc":"AddedUpdated tzdata to 1.0.1Vietnamese/Czech/Hebrew/Bosnian translationsFormatter settings, and ran the formatter on the codebaseFixedRomanian translation fixesVarious documentation/typespec fixesParser for ISOweek#559#577","ref":"changelog.html#3-6-2","title":"Change Log - 3.6.2","type":"extras"},{"doc":"Potentially BreakingRequire Elixir v1.6+AddedSetup property based test framework and add sample tests (#480)Turkish translation (#534)Add details to parsing documentation (#540)Support time units in singular (#509)#538 documentation of default formatting directivesSupport for tzdata 1.0.0 (#536)FixedAddress handling of ambiguity in timezone conversions by using UTC clock time, where possible. See #488#491 gracefully handle errors resolving a timezone using wall clock time#514 gracefully handle converting date to datetime on tz boundary#531 address some issues with shifting over timezone boundaries#507 unnecessary tz conversion during logical shifts results in erroneous results#532 shift priority incorrectly inverted#537 precision not preserved in some situationsElixir 1.8 deprecationsTimex.diff/2 specTimex.Timezone.convert/2 when using custom time zonesLetter encoding on PT translationSwedish translation id for March","ref":"changelog.html#3-6-1","title":"Change Log - 3.6.1","type":"extras"},{"doc":"AddedTimex.set/2 now also accepts setting the :date from a %Date{} struct.Interval.difference/2 removes one interval from another","ref":"changelog.html#3-5-0","title":"Change Log - 3.5.0","type":"extras"},{"doc":"AddedAfrikaans translations (Julian Dicks)The :inclusive option for Timex.between?/4 :start and :end in addition to trueFixedstrftime_iso_kitchen no longer discards dates","ref":"changelog.html#3-4-1","title":"Change Log - 3.4.1","type":"extras"},{"doc":"NOTE: There are breaking changes in this release. This is not going to result in a major version bump, as the old behavior did not match the docs, and was incorrect to boot. In other words, the breaking changes only affect you if you relied on the incorrect behavior, if you were expecting the documented behavior, then these are not breaking changes.AddedInterval.contains?/2 to test if one interval contains anotherFixedInterval overlap was being improperly calculatedInterval behavior with respect to open/closed bounds was incorrect (open bounds were being treated as closed and vice versa)Intervals could be created with invalid from/until (i.e. creating an interval with an until before the from)Interval documentation improvements for clarity","ref":"changelog.html#3-4-0","title":"Change Log - 3.4.0","type":"extras"},{"doc":"AddedRomanian translations (Cezar Halmagean)Fixed#280 - formatting of non-ISO week numbers","ref":"changelog.html#3-1-13","title":"Change Log - 3.1.13","type":"extras"},{"doc":"Sorry I didn't keep this up to date, please review the commits in git for these versions.","ref":"changelog.html#3-1-6-12","title":"Change Log - 3.1.6-12","type":"extras"},{"doc":"AddedTranslations for zh_CN (Chinese)Translations for pl (Polish)#244 - Support for fractional offsetsFixedVarious documentation fixesFix #262 - Day of week calculation in Julian calendar - Mark MeeusFix #260 - Duration formatting edge case - Slava KiselFix #257 - Fix from_iso_triplet logic - Mathew BramsonFix #252 - Formatting/parsing of fractional seconds was not round-trippableFix #248 - Make fractional second formatting consistent with standard libraryAdd application callback so we can provide better errors if :tzdata isn't startedEnable fallback_to_any in Timex.Protocol","ref":"changelog.html#3-1-6","title":"Change Log - 3.1.6","type":"extras"},{"doc":"FixedFix #218 - Bug with Timex.from_iso_tripletZone abbreviations produced by parsing did not always match those in tzdata","ref":"changelog.html#3-1-5","title":"Change Log - 3.1.5","type":"extras"},{"doc":"FixedFix #214 - Permit any valid datetime type in formatting APIFix #215 - Properly validate 2-digit years","ref":"changelog.html#3-1","title":"Change Log - 3.1","type":"extras"},{"doc":"ChangedDuration.scale/2 now works with float coefficients","ref":"changelog.html#3-0-8","title":"Change Log - 3.0.8","type":"extras"},{"doc":"AddedImplementation of Timex.Protocol for Map. This is primarily useful (and intended) for dealing with deserialized date/time structs.FixedPrecision calculation for microseconds was sometimes incorrect.","ref":"changelog.html#3-0-7","title":"Change Log - 3.0.7","type":"extras"},{"doc":"AddedAdded parsing for ISO-8601 durations to the Duration APIFixed#206 - Bug in shifting DateTimes","ref":"changelog.html#3-0-6","title":"Change Log - 3.0.6","type":"extras"},{"doc":"Fixed#199 - Handling of timezone names with - was broken.","ref":"changelog.html#3-0-5","title":"Change Log - 3.0.5","type":"extras"},{"doc":"AddedAdd Duration.to_time/1, to_time!/1, from_time/1 for conversions to/from Time","ref":"changelog.html#3-0-3","title":"Change Log - 3.0.3","type":"extras"},{"doc":"FixedConverting a NaiveDateTime with to_datetime did not include microsecondsnow/0, now/1, local/0 were not microsecond precise","ref":"changelog.html#3-0-2","title":"Change Log - 3.0.2","type":"extras"},{"doc":"AddedDuration.to_clock/1 - convert a Duration to a {hour,min,sec,usec} tupleDuration.from_clock/1 - convert a {hour,min,sec,usec} tuple to a Duration","ref":"changelog.html#3-0-1","title":"Change Log - 3.0.1","type":"extras"},{"doc":"IMPORTANT: This release is a significant rewrite of Timex's internals as well as API. Many things have remained unchanged, but there are many things that have as well. Mostly the removal of prior deprecations, and the removal (without deprecation) of things incompatible with, or now redundant due to, the introduction of calendar types in Elixir 1.3 and their impact on Timex. The list of these changes will be comprehensively spelled out below, along with recommendations for alternatives in the cases of removals.Fixed#185#137AddedTimex.Protocol (defines the API all calendar types must implement to be used with Timex)compare/3, diff/3 shift/2, now allow the use of :milliseconds and :microsecondsset/2 now allow the use of :microsecondTimex.Durationto_gregorian_microseconds/1, converts a date/time value to microseconds since year zeroChangedTimex's old Date/DateTime types are replaced by Elixir 1.3's new calendar types, NaiveDateTime is now used where appropriate, and AmbiguousDateTime remains in order to handle timezone ambiguities.Timex.diff/3 now returns to it's old behaviour of returning a signed integer for values, so that diffing/comparing can be done on a single value.Renamed Timex.Time to Timex.Duration to better reflect it's purpose and prevent conflicts with Elixir's built-in Time type.Renamed Timex.Format.Time.* to Timex.Format.Duration.*Renamed :timestamp options to :durationRenamed *_timestamp functions to *_durationChanged Timex.Duration to operate on and return Duration structs rather than Erlang timestamp tuplesChanged Duration.from/2, to Duration.from_*/1, moving the unit into the name.Renamed to_erlang_datetime to to_erlRemovedTimex.Date (use Timex now)Timex.DateTime (use Timex now)Timex.Convertable (no longer makes sense in the face of differentiating NaiveDateTime/DateTime)set/2 no longer allow the use of :millisecondRemoved Timex.dateDeprecated Timex.datetimeRemoved from_timestamp functionsRemoved to_gregorianRemoved to_seconds/2 in favor of to_gregorian_seconds/1 and to_unix/1Removed normalize/1, it no longer is necessary. normalize/2 still exists however","ref":"changelog.html#3-0-0","title":"Change Log - 3.0.0","type":"extras"},{"doc":"FixedSome behaviour around shifting across DST boundaries was behaving incorrectly (#142)","ref":"changelog.html#2-1-3","title":"Change Log - 2.1.3","type":"extras"},{"doc":"This release adds the base for locale-awareness in Timex, including one locale (&quot;ru&quot;), support for formatting datetimes in another locale, and functions which return names of things will now use the default locale. To configure Timex's default locale, put the following in config.exs:config :timex, default_locale: &quot;ru&quot; # or whatever locale you wantAddedThe ability to configure a default locale (the default is &quot;en&quot; if no config is provided) for formatting/translation of stringsThe ability to format a string using a given locale, otherwise the default locale is usedTranslations for the &quot;ru&quot; locale, more to comeLocale awareness throughout the core API, so functions such as day_name will return the day name in the configured locale, if translations for that locale were providedTimex.lformat, and Timex.lformat! variants of the formatting functions, which take a locale to use in formattingAdded a relative time formatter, which functions very similarly to Moment.js's relative time formatting You can use it with Timex.format! or Timex.format, by providing :relative as the formatter, and using the {relative} token in your format string. NOTE: The relative formatter does not support other tokens, only {relative} for now, if it seems like there is a use case where {relative} should support other tokens, I'll consider adding that.Added Timex.from_now, which takes:A single Convertable, which returns the relative time between that date/time value and nowTwo Convertables, which returns the relative time between the first date/time value, and the reference date (provided as the 2nd)Two variants of the above which also take a locale as the last argument, and results in the string being translated to that localeAdded ASN.1 parsing/formatting supportChangedAll functions which return strings, and all formatting functions, will use the default locale for translation, which is &quot;en&quot; unless another was configured, or one was provided if the function takes a locale as an argument.FixedMilliseconds should be able to be fractional - the is_millisecond guard was only allowing integers","ref":"changelog.html#2-1-2","title":"Change Log - 2.1.2","type":"extras"},{"doc":"AddedImplementations of the Inspect protocol for Date, DateTime, AmbiguousDateTime, TimezoneInfo, and AmbiguousTimezoneInfoChangedWhen inspecting any Timex types, the compact view will be used, pass structs: false to view the raw data structures if needed.","ref":"changelog.html#2-1-1","title":"Change Log - 2.1.1","type":"extras"},{"doc":"AddedTwo new protocols, Timex.Comparable and Timex.Convertable, implementing these two for your own date/time types will allow you to use the Timex API with your own types, just be aware that only Dates, DateTimes, or AmbiguousDateTimes will be returned as date/time representations, but it should be trivial to add a function in your implementation to convert back.Basic Julian calendar implementation, which allows you to get the Julian date for a given Convertable, see Timex.to_julian/1, you can also get the day of the week number for a Julian date, via Timex.Calendar.Julianto_julian function to the Timex.Convertable protocolTimex.timezones to get a list of all valid timezonesChangedPOTENTIALLY BREAKING The {Zname} format token was formatting with the abbreviation, which is incorrect. It has been changed to format with the full name, use {Zabbr} if you want the abbreviation.Moved comparison and diffing behaviour into a new protocol, Timex.Comparable, which allows you to now provide your own implementations for comparing other date or datetime types against Timex ones. This makes the API more flexible, and also cleaned up the code quite a bit.Modified Timex API to accept Comparables for just about all functionsAdded implementations of Timex.Comparable for Tuple, Date, DateTime, AmbiguousDateTimeAdded implementations of Timex.Convertable for Map and Atom. The former will accept any map with either DateTime-like keys (i.e. year/month/day/hour/minute/etc.) as strings or atoms, or any Date-like keys (year/month/day), as strings or atoms. The latter will accept only two atoms which represent Dates/DateTimes right now, :epoch, and :zero.Modified Timex.Comparable to take any Timex.Convertable","ref":"changelog.html#2-1-0","title":"Change Log - 2.1.0","type":"extras"},{"doc":"A number of performance enhancements due to refactoring for Convertable and Comparable, particularly with diffingThe %Z strftime format token was formatting timezones as abbreviations, which is not round-trippable due to timezone ambiguity. This token now formats using the full timezone name, which is valid according to the strftime standard.The {Zname} token had the same problem as above, and has been fixed to use the full name","ref":"changelog.html#fixed","title":"Change Log - Fixed","type":"extras"},{"doc":"READ THIS: This release contains breaking changes from the previous API. The changes are easy to make, and the compiler will catch almost all of them. See the Migrating section of the README for details on migrating to 2.0AddedNew Date type, which is basically the same as DateTime, but without time/timezone, which works with most all API functions that work with DateTimes, except those which are specific to time or timezones. Functions which take a Date or DateTime and options which can manipulate both date and time properties, like set or shift will work as you'd expect with Date values and time-based properties (setting a time property will change nothing, shifting will work for values which represent at least a day, sub-day values will not change the Date).New AmbiguousDateTime type which is returned instead of DateTime when the datetime in question falls in an ambiguously defined timezone period. This type contains two fields, :before and :after which contain the DateTime values to choose from. This handles cases where previously an error would likely have been thrown, or the behaviour would've been undefined.Timex.date and Timex.datetime, which are the equivalent of the old Date.from, except produce a Date or DateTime respectively. The from API still exists in the Date and DateTime modules, and is present in the Timex API for easier migration, but will be deprecated in the future.New from_erl, from_timestamp, from_microseconds, from_milliseconds, from_seconds, from_days functions to replace the old from API, and to match the Time API more closely.to_date/to_datetime/to_unix/to_timestamp/to_gregorian_seconds conversions in Timex.Convertable (old Timex.Date.Convert protocol)before?/after?/between? functions to Timex moduleformat/format!/format_time/format_time!/parse/parse! API to Timex moduleweek_of_month/1 and /3 to get the week index of the month a date occurs inTimex.diff(this, other, :calendar_weeks) - get the diff between two dates in terms of weeks on the calendar, in other words, the diff is done based on the start of the earliest date's week, and the end of the latest date's weekChangedBREAKING All non-bang functions in the API now return error tuples of the form {:error, term} instead of raising exceptionsBREAKING All DateTime-related APIs can now return an AmbiguousDateTime, which must be handled by choosing which DateTime to use.BREAKING All Timezone lookups can now return an AmbiguousTimezoneInfo struct, which must be handled by choosing which TimezoneInfo to use.BREAKING DateTime.ms is now DateTime.millisecondBREAKING Date module API has moved to Timex moduleDate and DateTime modules now contain their respective implementations of the Timex API, all shared functions have moved to the Timex module. You can work with Date or DateTimes either through the Timex API, or through the API exposed in the Date and DateTime modules. It is recommended to use Timex though.BREAKING Renamed Timex.Date.Convert to Timex.ConvertableBREAKING diff/3 now returns the same value no matter which order the arguments are given. Use compare/3 to get the orderingDeprecatedTimex.from (old Date.from) variants. Use Timex.date, Timex.datetime, Date.from_&lt;type&gt;, or DateTime.from_&lt;type&gt; instead.DateTime.from. Use Timex.datetime, DateTime.from_erl, DateTime.from_&lt;type&gt;to_usecs, to_msecs, to_secs, and to_mins in favor of to_microseconds, to_milliseconds, to_seconds, and to_minutesAbbreviated unit names like secs or mins, sec or min, in favor of their full names (i.e. seconds, minutes, etc.)RemovedDateConvert/DateFormat aliasesFixedShifting datetimes is now far more accurate than it was previously, and handles timezone changes, ambiguous timezone periods, and non-existent time periods (such as those during a timezone change like DST).Diffing by weeks, months, and years is now much more preciseHumanized Time formatter now pluralizes units correctly","ref":"changelog.html#2-0-0","title":"Change Log - 2.0.0","type":"extras"},{"doc":"BREAKING: If you previously depended on parsing of timezone abbreviations for non-POSIX zones, for example, CEST, you will need to update your code to manually map that abbreviation to a valid zone name. Timezone abbreviations are only supported if they are POSIX timezones in the Olson timezone database.AddedAdded CHANGELOGAdd Date.from clause to handle Phoenix datetime_select changesetChangedTimezone abbreviation handling is now only valid for POSIX/Olson timezone names.Some small optimizationsDeprecatedN/ARemovedTimezone abbreviation handling for non-POSIX/Olson timezone namesFixedTimezone abbreviation handling (was previously non-deterministic/incorrect)Disable tzdata's auto-update during compilationUsage of imperative if","ref":"changelog.html#1-0-2","title":"Change Log - 1.0.2","type":"extras"},{"doc":"Basic UsageSome common scenarios with examples","ref":"basic-usage.html","title":"Basic Usage","type":"extras"},{"doc":"Getting the current datetime in UTCiex&gt; Timex.now #&lt;DateTime(2016-07-12T22:26:43Z Etc/UTC)&gt; iex&gt; Timex.today ~D[2016-07-12]Getting the current datetime in the local timezoneiex&gt; Timex.local #&lt;DateTime(2016-07-12T17:27:09-05:00 America/Chicago)&gt;Getting the current datetime in an arbitrary timezoneiex&gt; Timex.now(&quot;Europe/Copenhagen&quot;) #&lt;DateTime(2016-07-12T22:27:37+02:00 Europe/Copenhagen)&gt;","ref":"basic-usage.html#getting-the-date-time","title":"Basic Usage - Getting the date/time","type":"extras"},{"doc":"iex&gt; Timex.to_date({2015, 6, 24}) ~D[2015-06-24] iex&gt; Timex.to_datetime({{2015, 6, 24}, {4, 50, 34}}, &quot;America/Chicago&quot;) #&lt;DateTime(2015-06-24T04:50:34-05:00 America/Chicago)&gt; iex&gt; Timex.to_datetime({{2015, 6, 24}, {4, 50, 34}}, :local) #&lt;DateTime(2015-06-24T04:50:34-05:00 America/Chicago)&gt;","ref":"basic-usage.html#construction","title":"Basic Usage - Construction","type":"extras"},{"doc":"Parsing an ISO 8601-formatted DateTime string# With timezone offset iex&gt; Timex.parse!(&quot;2015-06-24T04:50:34-05:00&quot;, &quot;{ISO:Extended}&quot;) #&lt;DateTime(2015-06-24T04:50:34-05:00 Etc/GMT+05)&gt; # Without timezone offset iex&gt; Timex.parse!(&quot;2015-06-24T04:50:34Z&quot;, &quot;{ISO:Extended:Z}&quot;) #&lt;DateTime(2015-06-24T04:50:34Z Etc/UTC)&gt;","ref":"basic-usage.html#parsing-date-time-strings","title":"Basic Usage - Parsing date/time strings","type":"extras"},{"doc":"Formatting a DateTime as an ISO 8601 stringiex&gt; Timex.format!(Timex.to_datetime(~N[2015-06-24T00:04:09.293], &quot;America/Chicago&quot;), &quot;{ISO:Extended}&quot;) &quot;2015-06-24T00:04:09.293-05:00&quot; iex&gt; Timex.format!(Timex.to_datetime(~N[2015-06-24T00:04:09.293], &quot;America/Chicago&quot;), &quot;{ISO:Extended:Z}&quot;) &quot;2015-06-24T05:04:13.293Z&quot;","ref":"basic-usage.html#formatting-datetimes","title":"Basic Usage - Formatting DateTimes","type":"extras"},{"doc":"iex&gt; use Timex ...&gt; event = Timex.to_datetime({{2016, 6, 24}, {0, 0, 0}}) ...&gt; other_event = Timex.to_datetime({{2010, 1, 1}, {0, 0, 0}}) ...&gt; from = Timex.to_datetime({{2015, 1, 1}, {0, 0, 0}}) ...&gt; until = Timex.to_datetime({{2018, 1, 1}, {0, 0, 0}}) ...&gt; interval = Timex.Interval.new(from: from, until: until) ...&gt; event in interval true ...&gt; other_event in interval false","ref":"basic-usage.html#testing-if-one-event-occurs-in-an-interval","title":"Basic Usage - Testing if one event occurs in an interval","type":"extras"},{"doc":"Erlang Interop","ref":"erlang-interop.html","title":"Erlang Interop","type":"extras"},{"doc":"How to work with Erlang datetime and time representationsWithout Timex, you've probably been working with Erlang's standard library :calendar module and/or :os.timestamp function, you may have code which already works on them, or need to consume them from another library, etc. The two most common representations of time in Erlang are the datetime and timestamp tuples, {{year, month, day}, {hour, minute, second}}, and {megaseconds, seconds, microseconds} respectively. The former is of course used for representing dates and times in a familiar format, the latter is used for representing precise moments in time, down to the microsecond.Converting from Erlang datetime tuples# To bring the aliases for Timex&#39;s modules into scope, we need to &quot;use&quot; Timex iex&gt; use Timex # Our input datetime iex&gt; date = :calendar.universal_time {{2015, 6, 24}, {3, 59, 5}} # Conversions to various Elixir types iex&gt; Timex.to_datetime(date, &quot;Etc/UTC&quot;) ... iex&gt; Timex.to_naive_datetime(date) ... iex&gt; Timex.to_date(date) ...Converting from Erlang timestamp tuples# The simplest case, converting from a timestamp to a DateTime, using the highest precision iex&gt; Duration.from_erl(:os.timestamp()) #&lt;Duration(P46Y6M24DT23H44M56.846453S)&gt; # Alternatively if you want control over the precision (in this example, we only care about up-to-the-second precision): iex&gt; time |&gt; Duration.to_seconds |&gt; Timex.from_unix #&lt;DateTime(2015-06-24T04:18:33Z Etc/UTC)&gt;Converting DateTimes to Erlang datetime tuplesTimex.to_erl/1 converts any valid Timex date/datetime to an erlang date or datetime tuple.iex&gt; date = Timex.now ...&gt; Timex.to_erl(date) {{2015, 6, 24}, {4, 18, 33}}","ref":"erlang-interop.html#erlang-dates-and-times","title":"Erlang Interop - Erlang Dates and Times","type":"extras"},{"doc":"FormattingHow to format DateTimes as strings with Timex's DateFormat module, and time intervals with TimeFormatterFormatting DateTimesFormatting DateTimes in Timex is done via the Timex module. There are two built in formatters, :default and :strftime. The details of how to construct format strings can be found in the docs for Timex.Format.DateTime.Formatters.Default and Timex.Format.DateTime.Formatters.Strftime, so this document will be a brief overview of the DateFormat API and how you use it typically.# By default the :default formatter is used iex&gt; Timex.format({2013,8,18}, &quot;{YYYY}-{M}-{D}&quot;) {:ok, &quot;2013-8-18&quot;} # But you can use the :strftime formatter very easily iex&gt; Timex.format({2013,8,18}, &quot;%Y-%m-%d&quot;, :strftime) {:ok, &quot;2013-08-18&quot;} # If you create your own formatter, you can use it easily as well iex&gt; Timex.format({2013,8,18}, format_str, MyApp.MyDateFormatter) # If formatting fails for some reason you will get an `{:error, reason}` tuple, so it&#39;s # recommended to use `format/1` or `format/2`; however you can use the &quot;bang&quot; # versions of these two, `format!/1` or `format!/2` which will return the result directly, # or raise on failure iex&gt; Timex.format!({2013,8,18}, &quot;%Y-%m-%d&quot;, :strftime) &quot;2013-08-18&quot;Formatting durationsFormatting durations (or timestamps really) is done via the Timex.Format.Duration.Formatter module, or more commonly via Timex.format_duration. It is extensible like date/time formatting as well.# Time since the epoch iex&gt; Duration.now |&gt; Timex.format_duration &quot;P45Y7M25DT18H13M10.966072S&quot; iex&gt; Duration.now |&gt; Timex.format_duration(:humanized) &quot;45 years, 7 months, 3 weeks, 4 days, 18 hours, 13 minutes, 16 seconds, 141.422 milliseconds&quot; # Time it took to execute some code iex&gt; {interval, _} = Duration.measure(fn -&gt; 1..10000 |&gt; Enum.reverse end) #&lt;Duration(PT0.002614S)&gt; iex&gt; interval |&gt; Timex.format_duration &quot;PT0.002614S&quot; iex&gt; interval |&gt; Timex.format_duration(:humanized) &quot;2.614 milliseconds&quot;","ref":"formatting.html","title":"Formatting","type":"extras"},{"doc":"ParsingNOTE: While Timex is strict about handling user input in general, it makes no guarantees that you will be protected from malicious input, as there are some limitations when considering user-provided input to parsers. A few parsers in Timex accept unbounded input, such as {s-epoch}, because there is no defined limit on the number of seconds since UNIX epoch (you may want to represent a date far in the future for example). It's important that you do your own validation of user input before passing it to Timex for parsing as a second line of defense - for example, limiting the length of input strings to some reasonable max length. It's also highly recommended that you take care to be as restrictive with your format strings as possible.How to use Timex for parsing DateTimes from stringsParsing a DateTime from an input string is very similar to formatting them, and is quite simple:# Simple date format, default parser iex&gt; Timex.parse(&quot;2013-03-05&quot;, &quot;{YYYY}-{0M}-{0D}&quot;) # Simple date format, strftime parser iex&gt; Timex.parse(&quot;2013-03-05&quot;, &quot;%Y-%m-%d&quot;, :strftime) # Parse a date using the default parser and the shortcut directive for RFC 1123 iex&gt; Timex.parse!(&quot;Tue, 05 Mar 2013 23:25:19 Z&quot;, &quot;{RFC1123z}&quot;) #&lt;DateTime(2013-03-05T23:25:19Z Etc/UTC)&gt; # Or you can use the :strftime parser (note how without timezone information, a NaiveDateTime is returned) iex&gt; Timex.parse!(&quot;2013-03-05&quot;, &quot;%Y-%m-%d&quot;, :strftime) ~N[2013-03-05T00:00:00] # Any preformatted directive ending in `z` will shift the date to UTC/Zulu iex&gt; Timex.parse(&quot;Tue, 06 Mar 2013 01:25:19 +0200&quot;, &quot;{RFC1123}&quot;) {:ok, #&lt;DateTime(2013-03-05T23:25:19Z Etc/UTC)&gt;},You can also use &quot;bang&quot; versions (i.e. parse!), which will raise on failure rather than returning an {:error, reason} tuple (parse!/1 and parse!/2 respectively).Supported Standards and Common FormatsRFC822# RFC822 iex&gt; Timex.parse(&quot;Mon, 05 Jun 14 23:20:59 UTC&quot;, &quot;{RFC822}&quot;) {:ok, #DateTime&lt;2014-06-05 23:20:59Z&gt;} # RFC822z iex&gt; Timex.parse(&quot;Mon, 05 Jun 14 23:20:59 +00:00&quot;, &quot;{RFC822z}&quot;) {:ok, #DateTime&lt;2014-06-05 23:20:59Z&gt;}RFC3339# RFC3339 iex&gt; Timex.parse(&quot;2013-03-05T23:25:19+02:00&quot;, &quot;{RFC3339}&quot;) {:ok, #DateTime&lt;2013-03-05 23:25:19+02:00 +02 Etc/GMT-2&gt;} # RFC3339z iex&gt; Timex.parse(&quot;2013-03-05T23:25:19Z&quot;, &quot;{RFC3339z}&quot;) {:ok, #DateTime&lt;2013-03-05 23:25:19Z&gt;}RFC1123# RFC1123 iex&gt; Timex.parse(&quot;Tue, 05 Mar 2013 23:25:19 EST&quot;, &quot;{RFC1123}&quot;) {:ok, #DateTime&lt;2013-03-05 23:25:19-05:00 EST EST&gt;} # RFC1123z iex&gt; Timex.parse(&quot;Tue, 06 Mar 2013 01:25:19 Z&quot;, &quot;{RFC1123z}&quot;) {:ok, #DateTime&lt;2013-03-06 01:25:19Z&gt;}ASN1:GeneralizedTime # ASN1:GeneralizedTime iex&gt; Timex.parse(&quot;20090305232519&quot;, &quot;{ASN1:GeneralizedTime}&quot;) {:ok, ~N[2009-03-05 23:25:19]} # ASN1:GeneralizedTime:Z iex&gt; Timex.parse(&quot;20090305232519.456Z&quot;, &quot;{ASN1:GeneralizedTime:Z}&quot;) {:ok, #DateTime&lt;2009-03-05 23:25:19.456Z&gt;} # ASN1:GeneralizedTime:TZ iex&gt; Timex.parse(&quot;20090305232519.000-0700&quot;, &quot;{ASN1:GeneralizedTime:TZ}&quot;) {:ok, #DateTime&lt;2009-03-05 23:25:19-07:00 -07 Etc/GMT+7&gt;}ASN1:UTCtime # ASN1: UTCtime iex(26)&gt; Timex.parse(&quot;130305232519Z&quot;, &quot;{ASN1:UTCtime}&quot;) {:ok, #DateTime&lt;2013-03-05 23:25:19Z&gt;}ISO:Extended# ISO8601 (Extended) iex&gt; Timex.parse(&quot;2014-08-14T12:34:33+00:00&quot;, &quot;{ISO:Extended}&quot;) {:ok, #DateTime&lt;2014-08-14 12:34:33Z&gt;} iex&gt; Timex.parse(&quot;2014-08-14T12:34:33+0000&quot;, &quot;{ISO:Extended}&quot;) {:ok, #DateTime&lt;2014-08-14 12:34:33Z&gt;} iex&gt; Timex.parse(&quot;2014-08-14T12:34:33Z&quot;, &quot;{ISO:Extended:Z}&quot;) {:ok, #DateTime&lt;2014-08-14 12:34:33Z&gt;} # ISO8601 (Basic) iex&gt; Timex.parse(&quot;20140814T123433-0000&quot;, &quot;{ISO:Basic}&quot;) {:ok, #DateTime&lt;2014-08-14 12:34:33+00:00 GMT Etc/GMT+0&gt;} iex&gt; Timex.parse(&quot;20140814T123433Z&quot;, &quot;{ISO:Basic:Z}&quot;) {:ok, #DateTime&lt;2014-08-14 12:34:33Z&gt;} ISOdateiex&gt; Timex.parse(&quot;2007-08-13&quot;, &quot;{ISOdate}&quot;) {:ok, ~N[2007-08-13 00:00:00]}Default Directive FormatThis is a formatting language designed specifically alongside Timex, and its primary purpose and advantage is that it is simple and easy to memorize and read in comparison to the standard C stftime format.The full documentation on the set of directives and formatting options can be found here.Strftime Directive FormatThis is the standard C library formatting language used with the stftime function in C. Timex supports this as a familiar alternative to its native &quot;default&quot; formatting language as described above.This implementation in Elixir is mostly compatible with strftime. The exception is the absence of locale-depended results. All directives that imply textual result will produce English names and abbreviations.You can find the full documentation on this format and its options here.Relative FormatThis is a human readable relative date/time format that consists of a single directive, {relative}, and produces values like &quot;a few seconds ago&quot;, or &quot;2 days ago&quot;.You can find the full chart of mappings from ranges to values here.","ref":"parsing.html","title":"Parsing","type":"extras"},{"doc":"Frequently Asked QuestionsThese are questions asked at one point or another.How is Timex licensed?Timex is made available under the MIT license. See the GitHub repo for details.Why can't I compare dates and times with &lt;, &gt;, etc.?It is not possible to override the comparison operators to properly compare arbitrary structs. Use Timex.compare/2 or /3 to compare them instead.Which functions provide microsecond precision?All of them. Unless you convert to something with lower precision.How do I find the time interval between two dates?Use Timex.diff to obtain the number of microseconds, milliseconds, seconds, minutes, hours, days, months, weeks, or years between two dates.What is the support for timezones?Full support for retrieving local timezone configuration on OSX, *NIX, and Windows, conversion to any timezone in the Olson timezone database, and full support for timezone transitions.Timezone support is also exposed via the Timezone, and Timezone.Local modules. Their functionality is also exposed via the Date module's API, and most common use cases shouldn't need to access the Timezone namespace directly, but it's there if needed.","ref":"faq.html","title":"Frequently Asked Questions","type":"extras"},{"doc":"Using with EctoHow to use Timex DateTimes with Ecto.Getting StartedTimex has can be integrated with Ecto via the timex_ecto plugin which is available on hex.pm:defp deps do [{:timex, &quot;~&gt; x.x.x&quot;}, {:timex_ecto, &quot;~&gt; x.x.x&quot;}] endAvailable TypesTimex-Ecto exposes a few different types for you to use:Timex.Ecto.Date: An ISO date (YYYY-MM-DD)Timex.Ecto.Time: An ISO time (hh:mm:ss.sss)Timex.Ecto.DateTime: An ISO 8601 datetime in UTCTimex.Ecto.DateTimeWithTimezone: Same as DateTime, but contains the timezone, i.e. America/Chicago as well. NOTE currently this is only supported with PostgreSQL, as it relies on complex types which are not currently supported in MySQL, and SQL Server user defined types require CLR types backing them which I have not explored in depth as of yet. See the section below titled Using DateTimeWithTimezone for details.Model DefinitionIn order to use the Timex DateTime type instead of the Ecto DateTime type, your model should look something like the following:defmodule User do use Ecto.Model schema &quot;users&quot; do field :name, :string field :a_date, Timex.Ecto.Date # Timex version of :date, will reify as a Date field :a_time, Timex.Ecto.Time # Timex version of :time, will reify as a Time field :a_datetime, Timex.Ecto.DateTime # Timex version of :datetime, will reify as a NaiveDateTime field :a_datetimetz, Timex.Ecto.DateTimeWithTimezone # A custom datatype (:datetimetz) implemented by Timex, will reify as a DateTime end endUsing Timex with Ecto's timestamps macroSuper simple! Your timestamps will now be DateTime structs instead of Ecto.DateTime structs.defmodule User do use Ecto.Model use Timex.Ecto.Timestamps schema &quot;users&quot; do field :name, :string timestamps end endUsing with PhoenixPhoenix allows you to apply defaults globally to Ecto models via web/web.ex by changing the model function like so:def model do quote do use Ecto.Model use Timex.Ecto.Timestamps end endBy doing this, you bring the Timex timestamps into scope in all your models.Using DateTimeWithTimezoneNOTE: This currently only applies to PostgreSQL.You must run the following SQL against the database you plan on using this type with:CREATE TYPE datetimetz AS ( dt timestamptz, tz varchar );You can then use this type like so:CREATE TABLE example ( id integer, created_at datetimetz );That's it!Full ExampleThe following is a simple test app I built for vetting this plugin:defmodule EctoTest.Repo do use Ecto.Repo, otp_app: :timex_ecto_test end defmodule EctoTest.User do use Ecto.Model use Timex.Ecto.Timestamps schema &quot;users&quot; do field :name, :string field :date_test, Timex.Ecto.Date field :time_test, Timex.Ecto.Time field :datetime_test, Timex.Ecto.DateTime field :datetimetz_test, Timex.Ecto.DateTimeWithTimezone end end defmodule EctoTest do import Ecto.Query use Timex alias EctoTest.User alias EctoTest.Repo def seed do time = Time.now date = Timex.today datetime = Timex.now datetimetz = Timezone.convert(datetime, &quot;Europe/Copenhagen&quot;) u = %User{name: &quot;Paul&quot;, date_test: date, time_test: time, datetime_test: datetime, datetimetz_test: datetimetz} Repo.insert!(u) end def all do query = from u in User, select: u Repo.all(query) end end defmodule EctoTest.App do use Application def start(_type, _args) do import Supervisor.Spec tree = [worker(EctoTest.Repo, [])] opts = [name: EctoTest.Sup, strategy: :one_for_one] Supervisor.start_link(tree, opts) end endAnd the results:iex(1)&gt; EctoTest.seed 14:45:43.461 [debug] INSERT INTO &quot;users&quot; (&quot;date_test&quot;, &quot;datetime_test&quot;, &quot;datetimetz_test&quot;, &quot;name&quot;, &quot;time_test&quot;) VALUES ($1, $2, $3, $4, $5) RETURNING &quot;id&quot; [{2015, 6, 25}, {{2015, 6, 25}, {19, 45, 43, 457000}}, {{{2015, 6, 25}, {21, 45, 43, 457000}}, &quot;Europe/Copenhagen&quot;}, &quot;Paul&quot;, {19, 45, 43, 457000}] OK query=3.9ms %EctoTest.User{__meta__: %Ecto.Schema.Metadata{source: &quot;users&quot;, state: :loaded}, date_test: ~D[2015-06-25], datetime_test: #&lt;DateTime(2015-06-25T21:45:43.457Z Etc/UTC)&gt;, datetimetz_test: #&lt;DateTime(2015-06-25T21:45:43.457+02:00 Europe/Copenhagen)&gt;, name: &quot;Paul&quot;, time_test: #&lt;Duration(P45Y6M6DT19H45M43.456856S)&gt; iex(2)&gt; EctoTest.all 14:45:46.721 [debug] SELECT u0.&quot;id&quot;, u0.&quot;name&quot;, u0.&quot;date_test&quot;, u0.&quot;time_test&quot;, u0.&quot;datetime_test&quot;, u0.&quot;datetimetz_test&quot; FROM &quot;users&quot; AS u0 [] OK query=0.7ms [%EctoTest.User{__meta__: %Ecto.Schema.Metadata{source: &quot;users&quot;, state: :loaded}, date_test: ~D[2015-06-25], datetime_test: #&lt;DateTime(2015-06-25T21:45:43.457Z Etc/UTC)&gt;, datetimetz_test: #&lt;DateTime(2015-06-25T21:45:43.457+02:00 Europe/Copenhagen)&gt;, name: &quot;Paul&quot;, time_test: #&lt;Duration(P45Y6M6DT19H45M43.456856S)&gt;}] iex(3)&gt;And that's all there is to it!","ref":"using-with-ecto.html","title":"Using with Ecto","type":"extras"},{"doc":"Custom ParsersHow to add custom DateTime parsers to TimexIt is unlikely you will need to write a custom parser for Timex, but should you be in such a position, you can easily plug in your own without much trouble.","ref":"custom-parsers.html","title":"Custom Parsers","type":"extras"},{"doc":"In brief, all you need to know is the following:Extend the Tokenizer behavior, by adding use Timex.Parse.DateTime.Tokenizer to the top of your module.Implement tokenize/1 callback.Implement apply/3 callback.What we are doing with the above is implementing a tokenizer for the format strings your custom parser will use. Incidentally this also is a prerequisite for implementing a custom formatter (you need to tokenize the format strings). Your best reference is to look at the two built-in tokenizers in Timex, as they are robust and complete implementations, but for the sake of an arbitrary example, let's walk through implementing a very simple parser for humanized strings like &quot;5 days before the fifth of July, 2015&quot;, where the following tokens are allowed:&quot;{shift}&quot; which should be in the form of &quot;&lt;integer&gt; &lt;unit&gt; &lt;before | after&gt;&quot;&quot;{day}&quot; of the form &quot;first&quot;, &quot;second&quot;, &quot;third&quot;, etc.&quot;{month}&quot; which is the full name of a month, i.e. &quot;July&quot;&quot;{year}&quot; which is the full four digit year","ref":"custom-parsers.html#getting-started","title":"Custom Parsers - Getting Started","type":"extras"},{"doc":"Implementing the Humanized tokenizerWe start by defining our empty module:defmodule MyApp.DateTimeTokenizers.Humanized do use Timex.Parse.DateTime.Tokenizer endCompiling with this will produce the following errors:../humanized.ex:1: warning: undefined behaviour function tokenize/1 (for behaviour Timex.Parse.DateTime.Tokenizer) ../humanized.ex:1: warning: undefined behaviour function apply/3 (for behaviour Timex.Parse.DateTime.Tokenizer)So we need to implement the tokenize function which takes a format string and produces a list of Directive structs (or {:error, term}). The following implementation makes use of Combine, a dependency pulled in by Timex for parsing tasks, and while you do not need to implement your tokenizer using Combine, the parser function given to each Directive must take a single argument of %Combine.ParserState{}, and return it, updated with the status and results from parsing. See the Combine repo for examples on how to implement these parsers (it is really rather trivial). My recommendation is to simply use Combine, as it is well suited for these tasks, but now you know how to work around it if so desired.defmodule MyApp.DateTimeTokenizers.Humanized do use Timex.Parse.DateTime.Tokenizer use Combine @days [ &quot;first&quot;, &quot;second&quot;, &quot;third&quot;, &quot;fourth&quot;, &quot;fifth&quot;, &quot;sixth&quot;, &quot;seventh&quot;, &quot;eighth&quot;, &quot;ninth&quot;, &quot;tenth&quot;, &quot;eleventh&quot;, &quot;twelfth&quot;, &quot;thirteenth&quot;, &quot;fourteenth&quot;, &quot;fifteenth&quot;, &quot;sixteenth&quot;, &quot;seventeenth&quot;, &quot;eighteenth&quot;, &quot;nineteenth&quot;, &quot;twentieth&quot;, &quot;twenty-first&quot;, &quot;twenty-second&quot;, &quot;twenty-third&quot;, &quot;twenty-fourth&quot;, &quot;twenty-fifth&quot;, &quot;twenty-sixth&quot;, &quot;twenty-seventh&quot;, &quot;twenty-eighth&quot;, &quot;twenty-ninth&quot;, &quot;thirtieth&quot;, &quot;thirty-first&quot; ] def tokenize(s) do case Combine.parse(s, parser) do results when is_list(results) -&gt; directives = results |&gt; List.flatten |&gt; Enum.filter(fn x -&gt; x !== nil end) case Enum.any?(directives, fn %Directive{type: type} -&gt; type != :literal end) do false -&gt; {:error, &quot;Invalid format string, must contain at least one directive.&quot;} true -&gt; {:ok, directives} end {:error, _} = err -&gt; err end end # Token parser defp parser do many1(choice([ between(char(?{), map(one_of(word, [&quot;shift&quot;, &quot;day&quot;, &quot;month&quot;, &quot;year&quot;]), &amp;map_directive/1), char(?})), map(none_of(char, [&quot;{&quot;, &quot;}&quot;]), &amp;map_literal/1) ])) end # Gets/builds the Directives for a given token defp map_directive(&quot;year&quot;), do: Directive.get(:year4, &quot;year&quot;) defp map_directive(&quot;month&quot;), do: Directive.get(:mfull, &quot;month&quot;) defp map_directive(&quot;day&quot;), do: %Directive{type: :oday_phonetic, value: &quot;day&quot;, parser: oday_phoenetic_parser()} defp map_directive(&quot;shift&quot;), do: %Directive{type: :date_shift, value: &quot;shift&quot;, parser: date_shift_parser(), weight: 99} # Generates directives for literals defp map_literal([]), do: nil defp map_literal(literals) when is_list(literals), do: Enum.map(literals, &amp;map_literal/1) defp map_literal(literal), do: %Directive{type: :literal, value: literal, parser: char(literal)} # Parses a phonetic ordinal day string, i.e. third defp oday_phoenetic_parser() do map(one_of(word_of(~r/[\\w\\-]/), @days), fn day -&gt; [day: to_day(day)] end) end # Parses a date shift expression, i.e. 3 days after defp date_shift_parser() do map(sequence([ integer, skip(spaces), one_of(word, [&quot;seconds&quot;, &quot;minutes&quot;, &quot;hours&quot;, &quot;days&quot;, &quot;weeks&quot;, &quot;months&quot;, &quot;years&quot;]), skip(spaces), one_of(word, [&quot;before&quot;, &quot;after&quot;]) ]), fn [n, shift, &quot;before&quot;] -&gt; [date_shift: [{to_shift(shift), -n}]] [n, shift, &quot;after&quot;] -&gt; [date_shift: [{to_shift(shift), n}]] end) end defp to_shift(shift), do: String.to_atom(shift) # Get the ordinal day value based on the ordinal day name defp to_day(name), do: Enum.find_index(@days, fn (n) -&gt; n == name end) + 1 end","ref":"custom-parsers.html#tokenizer-implementation","title":"Custom Parsers - Tokenizer Implementation","type":"extras"},{"doc":"A couple of things to notice.Many parsing directives are already built in to Timex, you can see which ones exist by looking at the Directive module. Rather than re-implement parsing of month names and 4 digit years, we're using Directive.get to pull the predefined Directives for those.However we also have two custom directives to handle the phonetic ordinal names and the date shift expressions, we're defining those directives (and their associated parsers) by hand. This is also the reason why you must implement apply/3. The built-in directives are applied via the parser, but custom tokens have to be applied by the tokenizer. I could have split out the tokenize and apply functions into two behaviours (say Tokenizer and Parser), but rather than force that upon you, this is a decision you can make in your own tokenizer (by using defdelegate to keep your apply/3 code separated).We're creating Directive structs for literal characters. This is important both for parsing and formatting, as it allows your parser to ignore context (i.e. skipping spaces, etc.) and focus on parsing the precise thing it needs to parse. When formatting it's also important, as it makes sure that we output a string which matches the format string precisely.The parsers for the directives return a keyword list of token: value as their result. This is a requirement, as the parser will take the input string and parse out a flattened list of {token, value} tuples. If your directive parsers do not produce values in this form, they will be ignored, and thus your parser will not work properly (and will likely produce an error).We're setting the weight key of the Directive for the shift expression. This will ensure that it is applied last. Consider the input string &quot;3 days after July fourth, 2015&quot;, if we try to apply the shift expression to the DateTime we get in apply/3 (which starts at 0/1/1T00:00:00), we will get an error for trying to shift the date out of the gregorian calendar, when really we want to apply the shift to the date specified later in the input string. By weighting the directive to be last, we will first apply the month, then the day, then the year to the initial DateTime, then apply the shift to the date we actually wanted it applied to. You can set the weight for all your directives to have them applied in a specific order, so keep it in mind for situations like this.We are left now with the responsibility of implementing apply/3, leaving our tokenizer implementation looking like the following:defmodule MyApp.DateTimeTokenizers.Humanized do use Timex.Parse.DateTime.Tokenizer use Combine alias Timex.Date ...snip... @doc &quot;&quot;&quot; Applies a token to the DateTime representing the current input string Only unrecognized tokens are applied via this function, standard tokens, such as :year4 will be handled by the parser itself. You can return {:ok, date}, {:error, reason}, or :unrecognized (if you don&#39;t know what to do with the provided token). &quot;&quot;&quot; def apply(%DateTime{} = date, token, value) do case token do :oday_phonetic -&gt; {:ok, %{date | :day =&gt; value}} :date_shift -&gt; case value do [{shift, n}] when is_integer(n) -&gt; {:ok, Timex.shift(date, [{shift, n}])} shift -&gt; {:error, &quot;Unrecognized shift operation: \#{Macro.to_string(shift)}&quot;} end _ -&gt; {:error, &quot;Unrecognized token: \#{token}.&quot;} end end ...snip... end","ref":"custom-parsers.html#implementation-notes","title":"Custom Parsers - Implementation Notes","type":"extras"},{"doc":"After all this, we're now ready to use our custom parser!&gt; alias MyApp.DateTimeTokenizers.Humanized &gt; phrase = &quot;3 days before the second of July, 2015&quot; &gt; format = &quot;{shift} the {day} of {month}, {year}&quot; &gt; Timex.parse!(phrase, format, Humanized) #&lt;DateTime(2015-07-01T00:00:00Z Etc/UTC)&gt;","ref":"custom-parsers.html#usage","title":"Custom Parsers - Usage","type":"extras"},{"doc":"Custom FormattersHow to implement a custom formatter for use with TimexImplementing your own custom formatter is pretty straightforward if you plan to use one of the built-in format string tokenizers (default or strftime). The following example will use a custom tokenizer, and implement the formatter over the top of that.The tokenizer is implemented as described in the Custom Parsers section, and the source code can be found in the Timex repo here.Getting StartedIn brief, all you need to know is the following:Extend the Formatter behavior, by adding use Timex.Format.DateTime.Formatter to the top of your module.Implement tokenize/1 callback.Implement format/2 callback.Implement format!/2 callback.The tokenize/1 callback can simply be delegated to the tokenizer you wish to use, while the implementation of format/2 and format!/2 are what we really are interested in here.Implementing the Humanized formatterThe implementation of the formatter for our &quot;humanized&quot; date format would like something like the following:defmodule MyApp.DateTimeFormatters.Humanized do use Timex.Format.DateTime.Formatter alias Timex.Format.FormatError alias MyApp.DateTimeTokenizers.Humanized, as: Tokenizer @days [ &quot;first&quot;, &quot;second&quot;, &quot;third&quot;, &quot;fourth&quot;, &quot;fifth&quot;, &quot;sixth&quot;, &quot;seventh&quot;, &quot;eighth&quot;, &quot;ninth&quot;, &quot;tenth&quot;, &quot;eleventh&quot;, &quot;twelfth&quot;, &quot;thirteenth&quot;, &quot;fourteenth&quot;, &quot;fifteenth&quot;, &quot;sixteenth&quot;, &quot;seventeenth&quot;, &quot;eighteenth&quot;, &quot;nineteenth&quot;, &quot;twentieth&quot;, &quot;twenty-first&quot;, &quot;twenty-second&quot;, &quot;twenty-third&quot;, &quot;twenty-fourth&quot;, &quot;twenty-fifth&quot;, &quot;twenty-sixth&quot;, &quot;twenty-seventh&quot;, &quot;twenty-eighth&quot;, &quot;twenty-ninth&quot;, &quot;thirtieth&quot;, &quot;thirty-first&quot; ] defdelegate tokenize(format_string), to: Tokenizer def format!(date, format_string) do case format(date, format_string) do {:ok, result} -&gt; result {:error, reason} -&gt; raise FormatError, message: reason end end def format(date, format_string) do case tokenize(format_string) do {:ok, []} -&gt; {:error, &quot;There were no formatting directives in the provided string.&quot;} {:ok, dirs} when is_list(dirs) -&gt; do_format(Timex.to_naive_datetime(date), dirs, &lt;&lt;&gt;&gt;) {:error, reason} -&gt; {:error, {:format, reason}} end end defp do_format(_date, [], result), do: {:ok, result} defp do_format(_date, _, {:error, _} = error), do: error defp do_format(date, [%Directive{type: :literal, value: char} | dirs], result) when is_binary(char) do do_format(date, dirs, &lt;&lt;result::binary, char::binary&gt;&gt;) end defp do_format(%NaiveDateTime{day: day} = date, [%Directive{type: :oday_phonetic} | dirs], result) do phonetic = Enum.at(@days, day - 1) do_format(date, dirs, &lt;&lt;result::binary, phonetic::binary&gt;&gt;) end defp do_format(date, [%Directive{type: :date_shift} | dirs], result) do do_format(date, dirs, &lt;&lt;result::binary, &quot;currently&quot;::binary&gt;&gt;) end defp do_format(date, [%Directive{type: type, modifiers: mods, flags: flags, width: width} | dirs], result) do case format_token(type, date, mods, flags, width) do {:error, _} = err -&gt; err formatted -&gt; do_format(date, dirs, &lt;&lt;result::binary, formatted::binary&gt;&gt;) end end endAs you can see the implementation is pretty straightforward. You'll notice that the last do_format implementation calls an imported function format_token/5, this allows you to delegate the formatting of known directives to the formatter, which will use standard formatting rules. You can of course override the formatting of directives using the style above (pattern matching on the directive type and handling the formatting directly).To use our new formatter with Timex.format:iex&gt; use Timex iex&gt; alias MyApp.DateTimeFormatters.Humanized, as: HumanFormat iex&gt; alias MyApp.DateTimeTokenizers.Humanized iex&gt; format = &quot;{shift} the {day} of {month}, {year}&quot; iex&gt; Timex.format(Timex.now, format, HumanFormat) {:ok, &quot;currently the eleventh of August, 2015&quot;}","ref":"custom-formatters.html","title":"Custom Formatters","type":"extras"}]